#!/usr/bin/env python

import re

from RouToolPa.Collections.General import IdList

from RouToolPa.Parsers.Abstract import Collection

from RouToolPa.Routines import AnnotationsRoutines, SequenceRoutines


class PrimerEntryPrimer3:
    def __init__(self, penalty, seq, start, length, melting_temperature, gc_content,
                 self_any_th, self_end_th, hairpin_th, end_stability):
        self.penalty = penalty                              #float
        self.seq = seq                                      #str
        self.start = start                                  #int
        self.length = length                                #int
        self.melting_temperature = melting_temperature      #float
        self.gc_content = gc_content                        #float
        self.self_any_th = self_any_th                      #float
        self.self_end_th = self_end_th                      #float
        self.hairpin_th = hairpin_th                        #float
        self.end_stability = end_stability                  #float


class PrimerPairEntryPrimer3:
    def __init__(self, pair_id=None, product_size=None, penalty=None, compl_any_th=None, compl_end_th=None,
                 left_primer=None, right_primer=None):
        self.id = pair_id                       #int
        self.product_size = product_size        #int
        self.penalty = penalty                  #float
        self.compl_any_th = compl_any_th        #float
        self.compl_end_th = compl_end_th        #float
        self.left_primer = left_primer          #PrimerEntryPrimer3
        self.right_primer = right_primer        #PrimerEntryPrimer3

    def primer_string(self):
        string = ""

        string += "PRIMER_LEFT_%i_PENALTY=%f\n" % (self.id, self.left_primer.penalty)
        string += "PRIMER_RIGHT_%i_PENALTY=%f\n" % (self.id, self.right_primer.penalty)
        string += "PRIMER_LEFT_%i_SEQUENCE=%s\n" % (self.id, self.left_primer.seq)
        string += "PRIMER_RIGHT_%i_SEQUENCE=%s\n" % (self.id, self.right_primer.seq)
        string += "PRIMER_LEFT_%i=%i,%i\n" % (self.id, self.left_primer.start, self.left_primer.length)
        string += "PRIMER_RIGHT_%i=%i,%i\n" % (self.id, self.right_primer.start, self.right_primer.length)
        string += "PRIMER_LEFT_%i_TM=%f\n" % (self.id, self.left_primer.melting_temperature)
        string += "PRIMER_RIGHT_%i_TM=%f\n" % (self.id, self.right_primer.melting_temperature)
        string += "PRIMER_LEFT_%i_GC_PERCENT=%f\n" % (self.id, self.left_primer.gc_content)
        string += "PRIMER_RIGHT_%i_GC_PERCENT=%f\n" % (self.id, self.right_primer.gc_content)
        string += "PRIMER_LEFT_%i_SELF_ANY_TH=%f\n" % (self.id, self.left_primer.self_any_th)
        string += "PRIMER_RIGHT_%i_SELF_ANY_TH=%f\n" % (self.id, self.right_primer.self_any_th)
        string += "PRIMER_LEFT_%i_SELF_END_TH=%f\n" % (self.id, self.left_primer.self_end_th)
        string += "PRIMER_RIGHT_%i_SELF_END_TH=%f\n" % (self.id, self.right_primer.self_end_th)
        string += "PRIMER_LEFT_%i_HAIRPIN_TH=%f\n" % (self.id, self.left_primer.hairpin_th)
        string += "PRIMER_RIGHT_%i_HAIRPIN_TH=%f\n" % (self.id, self.right_primer.hairpin_th)
        string += "PRIMER_LEFT_%i_END_STABILITY=%f\n" % (self.id, self.left_primer.end_stability)
        string += "PRIMER_RIGHT_%i_END_STABILITY=%f\n" % (self.id, self.right_primer.end_stability)

        return string

    def __str__(self):
        if self.id is None:
            return None
        string = "PRIMER_PAIR_%i_PENALTY=%f\n" % (self.id, self.penalty)

        string += self.primer_string()

        string += "PRIMER_PAIR_%i_COMPL_ANY_TH=%f\n" % (self.id, self.compl_any_th) if self.compl_any_th else ""
        string += "PRIMER_PAIR_%i_COMPL_END_TH=%f\n" % (self.id, self.compl_end_th) if self.compl_end_th else ""
        string += "PRIMER_PAIR_%i_PRODUCT_SIZE=%i\n" % (self.id, self.product_size)

        return string

    def pcr_product_seq(self, seq):
        if self.left_primer and self.right_primer:
            return seq[self.left_primer.start:self.right_primer.start+1]
        return None

    def gaps_inside_pcr_product(self, seq, min_gap_len=5):
        if self.left_primer and self.right_primer:
            pcr_seq = self.pcr_product_seq(seq)
            gap_reg_exp = re.compile("N{%i,}" % min_gap_len, re.IGNORECASE)
            gaps_list = list(gap_reg_exp.finditer(pcr_seq))
            if gaps_list:
                return True
            return False
        return None


class RecordPrimer3:
    def __init__(self, seq_id, seq, target_start, target_len,
                 pick_left_primer, pick_internal_oligo, pick_right_primer, product_size_range,
                 left_primer_count, internal_oligo_count, right_primer_count, primer_pair_count, primer_pair_list=[],
                 left_primer_choice_description=None, right_primer_choice_description=None,
                 internal_oligo_choice_description=None, pair_choice_description=None, chrom=None,
                 chrom_pos_start=None, chrom_pos_end=None, monomer=None, total_monomer_number=None):

        self.id = seq_id                                                                    # str

        self.seq_id = self.id.split(".primer")[0]

        self.seq = seq                                                                      # str
        self.target_start = target_start                                                    # int
        self.target_len = target_len                                                        # int
        self.pick_left_primer = pick_left_primer                                            # bool
        self.pick_internal_oligo = pick_internal_oligo                                      # bool
        self.pick_right_primer = pick_right_primer                                          # bool
        self.product_size_range = product_size_range                                        # (int, int)
        self.left_primer_count = left_primer_count                                          # int
        self.internal_oligo_count = internal_oligo_count                                    # int
        self.right_primer_count = right_primer_count                                        # int
        self.primer_pair_count = primer_pair_count                                          # int
        self.left_primer_choice_description = left_primer_choice_description                # str
        self.right_primer_choice_description = right_primer_choice_description              # str
        self.internal_oligo_choice_description = internal_oligo_choice_description          # str
        self.pair_choice_description = pair_choice_description                              # str
        self.primer_pair_list = primer_pair_list                                            # list of PrimerPairEntryPrimer3

        self.chrom = chrom                                                                  # str
        self.chrom_pos_start = chrom_pos_start                                              # int
        self.chrom_pos_end = chrom_pos_end                                                  # int

        self.monomer = monomer                                                              # str
        self.total_monomer_number = total_monomer_number                                    # float

    def __str__(self):
        string = ""
        string += "SEQUENCE_ID=%s\n" % self.id
        string += "SEQUENCE_TEMPLATE=%s\n" % self.seq
        string += "SEQUENCE_TARGET=%i,%i\n" % (self.target_start, self.target_len)
        string += "PRIMER_PICK_LEFT_PRIMER=%i\n" % (1 if self.pick_left_primer else 0) if not (self.pick_left_primer is None) else ""
        string += "PRIMER_PICK_INTERNAL_OLIGO=%i\n" % (1 if self.pick_internal_oligo else 0) if not (self.pick_internal_oligo is None) else ""
        string += "PRIMER_PICK_RIGHT_PRIMER=%i\n" % (1 if self.pick_right_primer else 0) if not (self.pick_right_primer is None) else ""
        string += "PRIMER_PRODUCT_SIZE_RANGE=%i-%i\n" % (self.product_size_range[0], self.product_size_range[1]) if self.product_size_range else ""
        string += "PRIMER_INTERNAL_OLIGO_EXPLAIN=%s\n" % self.left_primer_choice_description if not (self.left_primer_choice_description is None) else ""
        string += "PRIMER_PAIR_EXPLAIN=%s\n" % self.internal_oligo_choice_description if not (self.internal_oligo_choice_description is None) else ""
        string += "PRIMER_RIGHT_EXPLAIN=%s\n" % self.right_primer_choice_description if not (self.right_primer_choice_description is None) else ""
        string += "PRIMER_PAIR_EXPLAIN=%s\n" % self.pair_choice_description if not (self.pair_choice_description is None) else ""
        string += "PRIMER_LEFT_NUM_RETURNED=%i\n" % self.left_primer_count
        string += "PRIMER_RIGHT_NUM_RETURNED=%i\n" % self.right_primer_count
        string += "PRIMER_INTERNAL_NUM_RETURNED=%i\n" % self.internal_oligo_count
        string += "PRIMER_PAIR_NUM_RETURNED=%i\n" % self.primer_pair_count

        for primer_pair in self.primer_pair_list:
            string += str(primer_pair)

        string += "=\n"

        return string

    def correct_coordinates(self, sequence_dict):
        for primer_pair in self.primer_pair_list:
            primer_pair.left_primer.start = sequence_dict[self.seq_id].find(primer_pair.left_primer.seq)
            primer_pair.right_primer.start = sequence_dict[self.seq_id].find(SequenceRoutines.reverse_complement(primer_pair.right_primer.seq)) + primer_pair.right_primer.length - 1

    def remove_primers_with_gaps_in_pcr_product(self, min_gap_len=5):
        bad_primers_index_list = []

        good_pairs_list = []
        for primer_pair_index in range(0, len(self.primer_pair_list)):
            if self.primer_pair_list[primer_pair_index].gaps_inside_pcr_product(self.seq, min_gap_len=min_gap_len):
                bad_primers_index_list.append(primer_pair_index)
            else:
                good_pairs_list.append(self.primer_pair_list[primer_pair_index])

        ok_pairs = len(good_pairs_list)

        if bad_primers_index_list:
            if self.pair_choice_description:
                self.pair_choice_description += ", gaps in pcr product %i, ok after filtering %i" % (len(bad_primers_index_list), ok_pairs)
            self.primer_pair_list = []
            for good_pair_index in range(0, len(good_pairs_list)):
                good_pairs_list[good_pair_index].id = good_pair_index
            self.primer_pair_list = good_pairs_list
            self.primer_pair_count = ok_pairs

        return len(bad_primers_index_list)

    def table_form(self):
        string = ""
        metadata_location_str = "#Location"
        if self.primer_pair_list:
            metadata_str = "#SeqeunceID\t%s" % self.id
            if self.chrom:
                metadata_location_str += "\t%s" % self.chrom
                if self.chrom_pos_start and self.chrom_pos_end:
                    metadata_location_str += ":%i-%i" % (self.chrom_pos_start, self.chrom_pos_end)
            metadata_seq_str = "#Sequence\t%s" % self.seq
            header_str = "Primer_pair"
            pcr_product_size_str = "PCR product size"

            primer_pair_penalty_str = "Primer pair penalty"
            left_primer_penalty_str = "Left primer penalty"
            right_primer_penalty_str = "Right primer penalty"

            left_primer_seq_str = "Left primer seq"
            right_primer_seq_str = "Right primer seq"
            left_primer_tm_str = "Left primer melting temp"
            right_primer_tm_str = "Right primer melting temp"
            left_primer_gc_str = "Left primer GC,%"
            right_primer_gc_str = "Right primer GC,%"

            left_primer_start_str = "Left primer start(1-based)"
            right_primer_start_str = "Right primer start(1-based)"

            left_primer_len_str = "Left primer length"
            right_primer_len_str = "Right primer length"

            for primer_pair in self.primer_pair_list:
                header_str += "\t%s" % str(primer_pair.id)
                pcr_product_size_str += "\t%i" % primer_pair.product_size

                primer_pair_penalty_str += "\t%f" % primer_pair.penalty
                left_primer_penalty_str += "\t%s" % primer_pair.left_primer.penalty
                right_primer_penalty_str += "\t%s" % primer_pair.right_primer.penalty

                left_primer_seq_str += "\t%s" % primer_pair.left_primer.seq
                right_primer_seq_str += "\t%s" % primer_pair.right_primer.seq

                left_primer_tm_str += "\t%f" % primer_pair.left_primer.melting_temperature
                right_primer_tm_str += "\t%f" % primer_pair.right_primer.melting_temperature

                left_primer_gc_str += "\t%f" % primer_pair.left_primer.gc_content
                right_primer_gc_str += "\t%f" % primer_pair.right_primer.gc_content

                left_primer_start_str += "\t%i" % (primer_pair.left_primer.start + 1)
                right_primer_start_str += "\t%i" % (primer_pair.right_primer.start + 1)

                left_primer_len_str += "\t%i" % primer_pair.left_primer.length
                right_primer_len_str += "\t%i" % primer_pair.right_primer.length

            for table_string in (metadata_str, metadata_location_str, metadata_seq_str,
                                 header_str, pcr_product_size_str, primer_pair_penalty_str,
                                 left_primer_penalty_str, right_primer_penalty_str,
                                 left_primer_seq_str, right_primer_seq_str,
                                 left_primer_tm_str, right_primer_tm_str,
                                 left_primer_gc_str, right_primer_gc_str,
                                 left_primer_len_str, right_primer_len_str,
                                 left_primer_start_str, right_primer_start_str):

                string += table_string + "\n"

        return string

    def table_form2_short(self):
        """
        header = "#STR_ID\tScaffold\tStart\tStop\tMonomer\tCopy_number\tPRIMER_PAIR_NUMBER\tPRIMER_PAIR_ID\tPCR_PRODUCT_SIZE\tPRIMER_LEFT_TM\tPRIMER_RIGHT_TM\t" \
                 "PRIMER_LEFT_GC,%\tPRIMER_RIGHT_GC,%\tPRIMER_LEFT_SEQ\tPRIMER_RIGHT_SEQ\tTARGET_SEQ\n"
        """
        table_form2_short = ""
        for primer_pair in self.primer_pair_list:

            table_form2_short += "%s\t%s\t%i\t%i\t%s\t%s\t%i\t%i\t%i\t%.3f\t%.3f\t%.3f\t%.3f\t%s\t%s\n" % (self.id,
                                                                                                           self.chrom,
                                                                                                           self.chrom_pos_start if self.chrom_pos_start else self.target_start,
                                                                                                           self.chrom_pos_end if self.chrom_pos_end else self.target_start + self.target_len,
                                                                                                    str(self.monomer) if not(self.monomer is None) else ".",
                                                                                                    str(self.total_monomer_number) if not(self.total_monomer_number is None) else ".",
                                                                                                    self.primer_pair_count,
                                                                                                    primer_pair.id,
                                                                                                    primer_pair.product_size,
                                                                                                    primer_pair.left_primer.melting_temperature,
                                                                                                    primer_pair.right_primer.melting_temperature,
                                                                                                    primer_pair.left_primer.gc_content,
                                                                                                    primer_pair.right_primer.gc_content,
                                                                                                    primer_pair.left_primer.seq,
                                                                                                    primer_pair.right_primer.seq)
        return table_form2_short

    def table_form2(self):
        """
        header = "#STR_ID\tScaffold\tStart\tStop\tMonomer\tCopy_number\tPRIMER_PAIR_NUMBER\tPRIMER_PAIR_ID\tPCR_PRODUCT_SIZE\tPRIMER_LEFT_TM\tPRIMER_RIGHT_TM\t" \
                 "PRIMER_LEFT_GC,%\tPRIMER_RIGHT_GC,%\tPRIMER_LEFT_SEQ\tPRIMER_RIGHT_SEQ\tTARGET_SEQ\t" \
                 "PRIMER_LEFT_SELF_ANY_TH\tPRIMER_RIGHT_SELF_ANY_TH\tPRIMER_LEFT_SELF_END_TH\tPRIMER_RIGHT_SELF_END_TH\t" \
                 "PRIMER_LEFT_HAIRPIN_TH\tPRIMER_RIGHT_HAIRPIN_TH\tPRIMER_LEFT_END_STABILITY\tPRIMER_RIGHT_END_STABILITY\tREGION_SEQ\n"


        """
        table_form2 = ""
        for primer_pair in self.primer_pair_list:

            table_form2 += "%s\t%s\t%i\t%i\t%s\t%s\t%i\t%i\t%i\t%.3f\t%.3f\t%.3f\t%.3f\t%s\t%s\t%.3f\t%.3f\t" \
                           "%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%s\n" % (self.id,
                                                                         self.chrom,
                                                                         self.chrom_pos_start if self.chrom_pos_start else self.target_start,
                                                                         self.chrom_pos_end if self.chrom_pos_end else self.target_start + self.target_len,
                                                                         str(self.monomer) if not(self.monomer is None) else ".",
                                                                         str(self.total_monomer_number) if not(self.total_monomer_number is None) else ".",
                                                                         self.primer_pair_count,
                                                                         primer_pair.id,
                                                                         primer_pair.product_size,
                                                                         primer_pair.left_primer.melting_temperature,
                                                                         primer_pair.right_primer.melting_temperature,
                                                                         primer_pair.left_primer.gc_content,
                                                                         primer_pair.right_primer.gc_content,
                                                                         primer_pair.left_primer.seq,
                                                                         primer_pair.right_primer.seq,
                                                                         primer_pair.left_primer.self_any_th,
                                                                         primer_pair.right_primer.self_any_th,
                                                                         primer_pair.left_primer.self_end_th,
                                                                         primer_pair.right_primer.self_end_th,
                                                                         primer_pair.left_primer.hairpin_th,
                                                                         primer_pair.right_primer.hairpin_th,
                                                                         primer_pair.left_primer.end_stability,
                                                                         primer_pair.right_primer.end_stability,
                                                                         self.seq)

        return table_form2

    def alignments_string(self, segment_length=120, left_primer_symbol=">",
                          target_symbol="*", right_primer_symbol="<"):
        string = ""
        string += "#SeqeunceID\t%s\n" % self.id
        string += "#Location"
        if self.chrom:
            string += "\t%s" % self.chrom
            if self.chrom_pos_start and self.chrom_pos_end:
                string += ":%i-%i" % (self.chrom_pos_start, self.chrom_pos_end)
        string += "\n"
        #string += "#Sequence\t%s\n" % self.seq

        for primer_pair in self.primer_pair_list:
            string += "#Primer pair %i\n" % primer_pair.id
            string += "\n"

            location_list = [(primer_pair.left_primer.start,
                              primer_pair.left_primer.start + primer_pair.left_primer.length),

                             (self.target_start, self.target_start + self.target_len),

                             (primer_pair.right_primer.start - primer_pair.right_primer.length + 1,
                              primer_pair.right_primer.start + 1)]
            #print self.id
            #print location_list
            string += AnnotationsRoutines.draw_string_regions(self.seq, location_list,
                                                           [left_primer_symbol, target_symbol, right_primer_symbol],
                                                           overlap_symbol="#", line_per_record=False,
                                                           segment_length=segment_length,
                                                           num_of_spaces=3, num_of_space_lines=1, empty_symbol=" ")

        return string

    def table_form_and_alignments(self, segment_length=120, left_primer_symbol=">",
                          target_symbol="*", right_primer_symbol="<"):

        string = self.table_form()
        string += self.alignments_string(segment_length=segment_length, left_primer_symbol=left_primer_symbol,
                                         target_symbol=target_symbol, right_primer_symbol=right_primer_symbol)

        return string


class CollectionPrimer3(Collection):

    def __init__(self, record_list=None, primer3_file=None, from_file=True, id_based_location_dict=None, 
                 repeat_gff_file=None, id_description_entry="ID"):
        self.general_entry_list = ["SEQUENCE_ID",
                                   "SEQUENCE_TEMPLATE",
                                   "SEQUENCE_TARGET",
                                   "PRIMER_PICK_LEFT_PRIMER",
                                   "PRIMER_PICK_INTERNAL_OLIGO",
                                   "PRIMER_PICK_RIGHT_PRIMER",
                                   "PRIMER_PRODUCT_SIZE_RANGE",
                                   "PRIMER_LEFT_EXPLAIN",
                                   "PRIMER_RIGHT_EXPLAIN",
                                   "PRIMER_PAIR_EXPLAIN",
                                   "PRIMER_LEFT_NUM_RETURNED",
                                   "PRIMER_RIGHT_NUM_RETURNED",
                                   "PRIMER_INTERNAL_NUM_RETURNED",
                                   "PRIMER_PAIR_NUM_RETURNED"]

        self.primer_entry_prefix_list = ["PRIMER_LEFT", "PRIMER_RIGHT"]
        self.primer_entry_suffix_list = ["PENALTY",
                                         "SEQUENCE",
                                         "TM",
                                         "GC_PERCENT",
                                         "SELF_ANY_TH",
                                         "SELF_END_TH",
                                         "HAIRPIN_TH",
                                         "END_STABILITY"]

        self.primer_pair_prefix_list = ["PRIMER_PAIR"]
        self.primer_pair_suffix_list = ["PENALTY",
                                        "COMPL_ANY_TH",
                                        "COMPL_END_TH",
                                        "PRODUCT_SIZE"]

        id_based_dict = id_based_location_dict

        if repeat_gff_file:
            id_based_dict = AnnotationsRoutines.get_id_based_dict_from_gff(repeat_gff_file,
                                                                           id_entry=id_description_entry)

        if from_file:
            self.records = []
            with open(primer3_file, "r") as in_fd:
                for line in in_fd:
                    entry_dict = {}
                    lineeee = line
                    while lineeee[0] != "=":
                        line_list = lineeee.strip().split("=")
                        entry_dict[line_list[0]] = line_list[1]
                        lineeee = in_fd.next()

                    self._add_record(entry_dict, id_based_location_dict=id_based_dict)

        else:
            self.records = record_list

    def _add_record(self, entry_dict, id_based_location_dict=None):

        target_start, target_len = map(int, entry_dict["SEQUENCE_TARGET"].split(","))

        primer_pair_number = int(entry_dict["PRIMER_PAIR_NUM_RETURNED"]) if "PRIMER_PAIR_NUM_RETURNED" in entry_dict else None

        primer_pair_list = []
        
        if id_based_location_dict:
            #print id_based_location_dict
            chrom = id_based_location_dict[entry_dict["SEQUENCE_ID"]][0]
            start = int(id_based_location_dict[entry_dict["SEQUENCE_ID"]][1])
            end = int(id_based_location_dict[entry_dict["SEQUENCE_ID"]][2])

            monomer = id_based_location_dict[entry_dict["SEQUENCE_ID"]][3]["Pattern"][0] if "Pattern" in id_based_location_dict[entry_dict["SEQUENCE_ID"]][3] else None
            total_monomer_number = float(id_based_location_dict[entry_dict["SEQUENCE_ID"]][3]["N_copies"][0]) if "N_copies" in id_based_location_dict[entry_dict["SEQUENCE_ID"]][3] else None
        else:
            chrom = entry_dict["SEQUENCE_ID"]
            start = None
            end = None
            monomer = None
            total_monomer_number = None
        
        for primer_pair_index in range(0, primer_pair_number):
            left_primer_start, left_primer_len = map(int, entry_dict["PRIMER_LEFT_%i" % primer_pair_index].split(","))
            right_primer_start, right_primer_len = map(int, entry_dict["PRIMER_RIGHT_%i" % primer_pair_index].split(","))

            left_primer = PrimerEntryPrimer3(float(entry_dict["PRIMER_LEFT_%i_PENALTY" % primer_pair_index]),
                                             entry_dict["PRIMER_LEFT_%i_SEQUENCE" % primer_pair_index],
                                             left_primer_start,
                                             left_primer_len,
                                             float(entry_dict["PRIMER_LEFT_%i_TM" % primer_pair_index]),
                                             float(entry_dict["PRIMER_LEFT_%i_GC_PERCENT" % primer_pair_index]),
                                             float(entry_dict["PRIMER_LEFT_%i_SELF_ANY_TH" % primer_pair_index]),
                                             float(entry_dict["PRIMER_LEFT_%i_SELF_END_TH" % primer_pair_index]),
                                             float(entry_dict["PRIMER_LEFT_%i_HAIRPIN_TH" % primer_pair_index]),
                                             float(entry_dict["PRIMER_LEFT_%i_END_STABILITY" % primer_pair_index]))

            right_primer = PrimerEntryPrimer3(float(entry_dict["PRIMER_RIGHT_%i_PENALTY" % primer_pair_index]),
                                              entry_dict["PRIMER_RIGHT_%i_SEQUENCE" % primer_pair_index],
                                              right_primer_start,
                                              right_primer_len,
                                              float(entry_dict["PRIMER_RIGHT_%i_TM" % primer_pair_index]),
                                              float(entry_dict["PRIMER_RIGHT_%i_GC_PERCENT" % primer_pair_index]),
                                              float(entry_dict["PRIMER_RIGHT_%i_SELF_ANY_TH" % primer_pair_index]),
                                              float(entry_dict["PRIMER_RIGHT_%i_SELF_END_TH" % primer_pair_index]),
                                              float(entry_dict["PRIMER_RIGHT_%i_HAIRPIN_TH" % primer_pair_index]),
                                              float(entry_dict["PRIMER_RIGHT_%i_END_STABILITY" % primer_pair_index]))

            primer_pair_list.append(PrimerPairEntryPrimer3(pair_id=primer_pair_index,
                                                           product_size=int(entry_dict["PRIMER_PAIR_%i_PRODUCT_SIZE" % primer_pair_index]),
                                                           penalty=float(entry_dict["PRIMER_PAIR_%i_PENALTY" % primer_pair_index]),
                                                           compl_any_th=float(entry_dict["PRIMER_PAIR_%i_COMPL_ANY_TH" % primer_pair_index]) if "PRIMER_PAIR_%i_COMPL_ANY_TH" in entry_dict else None,
                                                           compl_end_th=float(entry_dict["PRIMER_PAIR_%i_COMPL_END_TH" % primer_pair_index]) if "PRIMER_PAIR_%i_COMPL_END_TH" in entry_dict else None,
                                                           left_primer=left_primer,
                                                           right_primer=right_primer))

        record = RecordPrimer3(entry_dict["SEQUENCE_ID"],
                               entry_dict["SEQUENCE_TEMPLATE"],
                               target_start, target_len,
                               int(entry_dict["PRIMER_PICK_LEFT_PRIMER"]) if "PRIMER_PICK_LEFT_PRIMER" in entry_dict else None,
                               int(entry_dict["PRIMER_PICK_INTERNAL_OLIGO"]) if "PRIMER_PICK_INTERNAL_OLIGO" in entry_dict else None,
                               int(entry_dict["PRIMER_PICK_RIGHT_PRIMER"]) if "PRIMER_PICK_RIGHT_PRIMER" in entry_dict else None,
                               map(int, entry_dict["PRIMER_PRODUCT_SIZE_RANGE"].split("-")) if "PRIMER_PRODUCT_SIZE_RANGE" in entry_dict else None,
                               int(entry_dict["PRIMER_LEFT_NUM_RETURNED"]) if "PRIMER_LEFT_NUM_RETURNED" in entry_dict else None,
                               int(entry_dict["PRIMER_INTERNAL_NUM_RETURNED"]) if "PRIMER_INTERNAL_NUM_RETURNED" in entry_dict else None,
                               int(entry_dict["PRIMER_RIGHT_NUM_RETURNED"]) if "PRIMER_RIGHT_NUM_RETURNED" in entry_dict else None,
                               primer_pair_number,
                               primer_pair_list=primer_pair_list,
                               left_primer_choice_description=entry_dict["PRIMER_LEFT_EXPLAIN"] if "PRIMER_LEFT_EXPLAIN" in entry_dict else None,
                               right_primer_choice_description=entry_dict["PRIMER_RIGHT_EXPLAIN"] if "PRIMER_RIGHT_EXPLAIN" in entry_dict else None,
                               internal_oligo_choice_description=entry_dict["PRIMER_INTERNAL_OLIGO_EXPLAIN"] if "PRIMER_INTERNAL_OLIGO_EXPLAIN" in entry_dict else None,
                               pair_choice_description=entry_dict["PRIMER_PAIR_EXPLAIN"] if "PRIMER_PAIR_EXPLAIN" in entry_dict else None,
                               chrom=chrom,
                               chrom_pos_start=start,
                               chrom_pos_end=end,
                               monomer=monomer,
                               total_monomer_number=total_monomer_number)

        self.records.append(record)

    def correct_coordinates(self, sequence_dict):
        for record in self.records:
            record.correct_coordinates(sequence_dict)

    def write(self, out_file):
        with open(out_file, "w") as out_fd:
            for record in self.records:
                out_fd.write(str(record))

    def write_table_form(self, out_file):
        with open(out_file, "w") as out_fd:
            for record in self.records:
                out_fd.write(record.table_form())

    def write_alignments(self, out_file, segment_length=120, left_primer_symbol=">",
                         target_symbol="*", right_primer_symbol="<"):
        with open(out_file, "w") as out_fd:
            for record in self.records:
                out_fd.write(record.alignments_string(segment_length=segment_length,
                                                      left_primer_symbol=left_primer_symbol,
                                                      target_symbol=target_symbol,
                                                      right_primer_symbol=right_primer_symbol))

    def write_table_form_with_alignments(self, out_file, segment_length=120, left_primer_symbol=">",
                                         target_symbol="*", right_primer_symbol="<"):
        with open(out_file, "w") as out_fd:
            for record in self.records:
                out_fd.write(record.table_form_and_alignments(segment_length=segment_length,
                                                              left_primer_symbol=left_primer_symbol,
                                                              target_symbol=target_symbol,
                                                              right_primer_symbol=right_primer_symbol))

    def write_table_form2_short(self, out_file):

        header = "#STR_ID\tScaffold\tStart\tStop\tMonomer\tCopy_number\tPRIMER_PAIR_NUMBER\tPRIMER_PAIR_ID" \
                 "\tPCR_PRODUCT_SIZE\tPRIMER_LEFT_TM\tPRIMER_RIGHT_TM\t" \
                 "PRIMER_LEFT_GC,%\tPRIMER_RIGHT_GC,%\tPRIMER_LEFT_SEQ\tPRIMER_RIGHT_SEQ\tTARGET_SEQ\n"

        with open(out_file, "w") as out_fd:
            out_fd.write(header)
            for record in self.records:
                out_fd.write(record.table_form2_short())

    def write_table_form2(self, out_file):

        header = "#STR_ID\tScaffold\tStart\tStop\tMonomer\tCopy_number\tPRIMER_PAIR_NUMBER\tPRIMER_PAIR_ID" \
                 "\tPCR_PRODUCT_SIZE\tPRIMER_LEFT_TM\tPRIMER_RIGHT_TM\t" \
                 "PRIMER_LEFT_GC,%\tPRIMER_RIGHT_GC,%\tPRIMER_LEFT_SEQ\tPRIMER_RIGHT_SEQ\t" \
                 "PRIMER_LEFT_SELF_ANY_TH\tPRIMER_RIGHT_SELF_ANY_TH\tPRIMER_LEFT_SELF_END_TH\tPRIMER_RIGHT_SELF_END_TH\t" \
                 "PRIMER_LEFT_HAIRPIN_TH\tPRIMER_RIGHT_HAIRPIN_TH\tPRIMER_LEFT_END_STABILITY\tPRIMER_RIGHT_END_STABILITY\tTARGET_SEQ\n"

        with open(out_file, "w") as out_fd:
            out_fd.write(header)
            for record in self.records:
                out_fd.write(record.table_form2())

    def filter_by_function(self, function):

        filtered_records, filtered_out_records = self.filter_records_by_function(function)

        return (CollectionPrimer3(record_list=filtered_records, primer3_file=None, from_file=False),
                CollectionPrimer3(record_list=filtered_out_records, primer3_file=None, from_file=False))

    def filter_out_records_without_primers(self):

        def record_with_primers(record):
            if record.primer_pair_count > 0:
                return True
            return False

        return self.filter_by_function(record_with_primers)

    def remove_primers_with_gaps_in_pcr_product(self, min_gap_len=5):
        for record in self.records:
            record.remove_primers_with_gaps_in_pcr_product(min_gap_len=min_gap_len)

    def seq_ids(self):
        id_list = IdList()

        for record in self.records:
            id_list.append(record.id)

        return id_list

    def extract_records_by_ids(self, id_list):
        extracted_record_list = []
        for record in self.records:
            if record.id in id_list:
                extracted_record_list.append(record)

        return CollectionPrimer3(from_file=False, record_list=extracted_record_list)





#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'
import os
import shutil
from RouToolPa.Tools.Abstract import Tool
from RouToolPa.Tools.LinuxTools import CGAS
from RouToolPa.Collections.General import IdList, SynDict

class HMMER3(Tool):
    """
    http://hmmer.janelia.org/
    """
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "hmmbuild", path=path, max_threads=max_threads)

    def hmmbuild(self, input, output, sample_name, summary_output="hmmbuild_summary.t",
                 data_type="protein", max_insert_len=None, window_length=None,
                 alt_model_constact_strategy=None, alt_rel_seq_weigthing_strategy=None,
                 alt_eff_weighting_strategy=None, alt_prior_strategy=None, ):

        options = " --cpu %i" % self.threads
        options += " --dna" if data_type == "dna" else " --rna" if data_type == "rna" else " --amino"
        options += " -n %s" % sample_name
        options += " --maxinsertlen %i" % max_insert_len if max_insert_len else ""
        options += " --w_length %i" % window_length if window_length else ""
        options += " --%s" % alt_model_constact_strategy if alt_model_constact_strategy else ""
        options += " --%s" % alt_rel_seq_weigthing_strategy if alt_rel_seq_weigthing_strategy else ""
        options += " --%s" % alt_eff_weighting_strategy if alt_eff_weighting_strategy else ""
        options += " --%s" % alt_prior_strategy if alt_prior_strategy else ""
        options += " -o %s" % summary_output
        options += " %s" % output
        options += " %s" % input

        self.execute(options, cmd="hmmbuild")

    def hmmpress(self, hmmfile):

        options = " %s" % hmmfile

        self.execute(options, cmd="hmmpress")

    def index_hmmfile(self, hmmfile):

        options = " --index %s" % hmmfile

        self.execute(options, cmd="hmmfetch")

    @staticmethod
    def extract_hits_by_query_ids(id_list, hit_file, output_file,
                                  fileformat="tblout", close_after_if_file_object=False):
        out_fd = output_file if isinstance(output_file, file) else open(output_file, "w")
        with open(hit_file, "r") as in_fd:
            if fileformat == "domtblout" or "tblout":
                if fileformat == "domtblout":
                    query_pos_in_file = 3
                elif fileformat == "tblout":
                    query_pos_in_file = 2
                for line in in_fd:
                    if line[0] == "#":
                        out_fd.write(line)
                        continue
                    try:
                        query_id = line.split()[query_pos_in_file]
                    except IndexError:
                        print(line)
                    if query_id in id_list:
                        out_fd.write(line)

        if (not isinstance(output_file, file)) or close_after_if_file_object:
            out_fd.close()

    def get_hmms_by_ids(self, hmmfile, id_file, output=None):

        options = " -f"
        options += " %s" % hmmfile
        options += " %s" % id_file
        options += " > %s" % output if output is not None else ""

        self.execute(options, cmd="hmmfetch")

    def get_single_hmm_by_id(self, hmmfile, hmm_id, output=None):

        options = "%s" % hmmfile
        options += " %s" % hmm_id
        options += " > %s" % output if output is not None else ""

        self.execute(options, cmd="hmmfetch")

    def hmmstat(self, hmm_file, stat_file=None):

        options = " %s" % hmm_file
        options += " > %s" % stat_file if stat_file is not None else ""

        self.execute(options, cmd="hmmstat")

    @staticmethod
    def get_ids_from_hmm3(hmmfile, ids_file=None, return_ids_list=False):
        """
        Extracts ids from hmm3 file:
            return_ids_list == True: captures output and returns ids_list, ids_file is ignored
            return_ids_list == False and ids_file == None: writes ids to stdout
            return_ids_list == False and ids_file != None: writes ids to ids_file

        """
        return CGAS.cgas(hmmfile, grep_pattern="NAME", whole_word_match=True, awk_code="{print $2}",
                         capture_output=return_ids_list, output=ids_file if not return_ids_list else None)

    def split_hmm(self, hmmfile, output_dir, num_of_recs_per_file, num_of_files=None, output_prefix=None, threads=4):

        try:
            os.mkdir(output_dir)
        except OSError:
            pass

        id_fd = CGAS.cgas(hmmfile, grep_pattern="NAME", whole_word_match=True, awk_code="{print $2}",
                          capture_output=True)

        split_index = 1
        ids_written = 0
        ids_list = IdList()
        #ids_list = read_ids(id_fd, close_after_if_file_object=False)
        ids_list.read(id_fd, close_after_if_file_object=True)
        number_of_ids = len(ids_list)
        out_prefix = self.split_filename(hmmfile)[1] if output_prefix is None else output_prefix

        num_of_ids = int(number_of_ids/num_of_files) + 1 if num_of_files else num_of_recs_per_file

        common_options = " -f"
        common_options += " %s" % hmmfile
        options_list = []
        while (ids_written + num_of_ids) <= number_of_ids:
            tmp_id_list = IdList(ids_list[ids_written:ids_written+num_of_ids])
            tmp_id_list.write("%s/%s_%i.ids" % (output_dir, out_prefix, split_index))

            options = common_options
            options += " %s/%s_%i.ids" % (output_dir, out_prefix, split_index)
            options += " > %s" % ("%s/%s_%i.hmm" % (output_dir, out_prefix, split_index))
            options_list.append(options)

            split_index += 1
            ids_written += num_of_ids

        if ids_written != number_of_ids:
            tmp_id_list = IdList(ids_list[ids_written:])
            tmp_id_list.write("%s/%s_%i.ids" % (output_dir, out_prefix, split_index))

            options = common_options
            options += " %s/%s_%i.ids" % (output_dir, out_prefix, split_index)
            options += " > %s" % ("%s/%s_%i.hmm" % (output_dir, out_prefix, split_index))
            options_list.append(options)

            split_index += 1
        #print options_list
        self.parallel_execute(options_list, cmd="hmmfetch", threads=threads)

    def __parse_hmmsxxx_common_options(self, tblout=None, domtblout=None, pfamtblout=None,
                                       dont_output_alignments=False, model_evalue_threshold=None,
                                       model_score_threshold=None,
                                       domain_evalue_threshold=None, domain_score_threshold=None,
                                       model_evalue_significant_threshold=None, model_score_significant_threshold=None,
                                       domain_evalue_significant_threshold=None,
                                       domain_score_significant_threshold=None,
                                       use_profile_GA_gathering_cutoffs_for_thresholds=False,
                                       use_profile_NC_noise_cutoffs_for_thresholds=False,
                                       use_profile_TC_trusted_cutoffs_for_thresholds=False,
                                       turn_off_all_heruristics=False, turn_off_bias_filter=False,
                                       MSV_threshold=None, Vit_threshold=None, Fwd_threshold=None,
                                       turn_off_biased_composition_score_corrections=None):
        #options = " --cpu %i" % self.threads
        #options += " -o %s" % outfile

        options = " --tblout %s" % tblout if tblout else ""
        options += " --domtblout %s" % domtblout if domtblout else ""
        options += " --pfamtblout %s" % pfamtblout if pfamtblout else ""
        options += " --noali" if dont_output_alignments else ""
        options += " --E %f" % model_evalue_threshold if model_evalue_threshold else ""
        options += " --T %f" % model_score_threshold if model_score_threshold else ""
        options += " --domE %f" % domain_evalue_threshold if domain_evalue_threshold else ""
        options += " --domT %f" % domain_score_threshold if domain_score_threshold else ""
        options += " --incE %f" % model_evalue_significant_threshold if model_evalue_significant_threshold else ""
        options += " --incT %f" % model_score_significant_threshold if model_score_significant_threshold else ""
        options += " --domincE %f" % domain_evalue_significant_threshold if domain_evalue_significant_threshold else ""
        options += " --domincT %f" % domain_score_significant_threshold if domain_score_significant_threshold else ""
        options += " --cut_ga" if use_profile_GA_gathering_cutoffs_for_thresholds else ""
        options += " --cut_nc" if use_profile_NC_noise_cutoffs_for_thresholds else ""
        options += " --cut_tc" if use_profile_TC_trusted_cutoffs_for_thresholds else ""
        options += " --max" if turn_off_all_heruristics else ""
        options += " --nobias" if turn_off_bias_filter else ""
        options += " --F1 %f" % MSV_threshold if MSV_threshold else ""
        options += " --F2 %f" % Vit_threshold if Vit_threshold else ""
        options += " --F3 %f" % Fwd_threshold if Fwd_threshold else ""
        options += " --nonull2" if turn_off_biased_composition_score_corrections else ""
        return options

    def hmmscan(self, hmmfile, seqfile, outfile, tblout=None, domtblout=None, pfamtblout=None,
                dont_output_alignments=False, model_evalue_threshold=None, model_score_threshold=None,
                domain_evalue_threshold=None, domain_score_threshold=None,
                model_evalue_significant_threshold=None, model_score_significant_threshold=None,
                domain_evalue_significant_threshold=None, domain_score_significant_threshold=None,
                use_profile_GA_gathering_cutoffs_for_thresholds=False,
                use_profile_NC_noise_cutoffs_for_thresholds=False,
                use_profile_TC_trusted_cutoffs_for_thresholds=False,
                turn_off_all_heruristics=False, turn_off_bias_filter=False,
                MSV_threshold=None, Vit_threshold=None, Fwd_threshold=None,
                turn_off_biased_composition_score_corrections=None,
                input_format=None):

        options = self.__parse_hmmsxxx_common_options(tblout=tblout, domtblout=domtblout,
                                                      pfamtblout=pfamtblout,
                                                      dont_output_alignments=dont_output_alignments,
                                                      model_evalue_threshold=model_evalue_threshold,
                                                      model_score_threshold=model_score_threshold,
                                                      domain_evalue_threshold=domain_evalue_threshold,
                                                      domain_score_threshold=domain_score_threshold,
                                                      model_evalue_significant_threshold=model_evalue_significant_threshold,
                                                      model_score_significant_threshold=model_score_significant_threshold,
                                                      domain_evalue_significant_threshold=domain_evalue_significant_threshold,
                                                      domain_score_significant_threshold=domain_score_significant_threshold,
                                                      use_profile_GA_gathering_cutoffs_for_thresholds=use_profile_GA_gathering_cutoffs_for_thresholds,
                                                      use_profile_NC_noise_cutoffs_for_thresholds=use_profile_NC_noise_cutoffs_for_thresholds,
                                                      use_profile_TC_trusted_cutoffs_for_thresholds=use_profile_TC_trusted_cutoffs_for_thresholds,
                                                      turn_off_all_heruristics=turn_off_all_heruristics,
                                                      turn_off_bias_filter=turn_off_bias_filter,
                                                      MSV_threshold=MSV_threshold, Vit_threshold=Vit_threshold,
                                                      Fwd_threshold=Fwd_threshold,
                                                      turn_off_biased_composition_score_corrections=turn_off_biased_composition_score_corrections)
        options += " --cpu %i" % self.threads
        options += " -o %s" % outfile
        options += " --qformat %s" if input_format else ""
        options += " %s" % hmmfile
        options += " %s" % seqfile

        self.execute(options, cmd="hmmscan")

    def parallel_hmmscan(self, hmmfile, seqfile, output_prefix, output_dir="./",
                         num_of_seqs_per_scan=None,
                         dont_output_alignments=False,
                         model_evalue_threshold=None, model_score_threshold=None,
                         domain_evalue_threshold=None, domain_score_threshold=None,
                         model_evalue_significant_threshold=None, model_score_significant_threshold=None,
                         domain_evalue_significant_threshold=None, domain_score_significant_threshold=None,
                         use_profile_GA_gathering_cutoffs_for_thresholds=False,
                         use_profile_NC_noise_cutoffs_for_thresholds=False,
                         use_profile_TC_trusted_cutoffs_for_thresholds=False,
                         turn_off_all_heruristics=False, turn_off_bias_filter=False,
                         MSV_threshold=None, Vit_threshold=None, Fwd_threshold=None,
                         turn_off_biased_composition_score_corrections=None,
                         input_format=None, threads=None,
                         combine_output_to_single_file=True,
                         biopython_165_compartibility=False,
                         extract_top_hits=True,
                         get_clusters_from_top_hits=True,
                         remove_tmp_dirs=True,
                         async_run=False, external_process_pool=None,
                         cpus_per_task=1,
                         handling_mode="local",
                         job_name=None,
                         log_prefix=None,
                         error_log_prefix=None,
                         max_running_jobs=None,
                         max_running_time=None,
                         max_memmory_per_cpu=None,
                         modules_list=None,
                         environment_variables_dict=None,
                         MAVR_scripts_dir="",
                         hmm_hit_parsing_mode="parse",
                         ):
        splited_fasta_dir = "%s/splited_fasta/" % output_dir
        splited_output_dir = "%s/splited_output/" % output_dir
        splited_tblout_dir = "%s/splited_tblout/" % output_dir
        splited_domtblout_dir = "%s/splited_domtblout/" % output_dir
        splited_pfamtblout_dir = "%s/splited_pfamtblout/" % output_dir
        splited_hit_info = "%s/splited_hit_info/" % output_dir

        directory_list = [
                         splited_fasta_dir,
                         splited_output_dir,
                         splited_tblout_dir,
                         splited_domtblout_dir,
                         splited_pfamtblout_dir,
                         splited_hit_info
                         ]

        for directory in directory_list:
            self.safe_mkdir(directory)

        common_options = self.__parse_hmmsxxx_common_options(tblout=None, domtblout=None,
                                                             pfamtblout=None,
                                                             dont_output_alignments=dont_output_alignments,
                                                             model_evalue_threshold=model_evalue_threshold,
                                                             model_score_threshold=model_score_threshold,
                                                             domain_evalue_threshold=domain_evalue_threshold,
                                                             domain_score_threshold=domain_score_threshold,
                                                             model_evalue_significant_threshold=model_evalue_significant_threshold,
                                                             model_score_significant_threshold=model_score_significant_threshold,
                                                             domain_evalue_significant_threshold=domain_evalue_significant_threshold,
                                                             domain_score_significant_threshold=domain_score_significant_threshold,
                                                             use_profile_GA_gathering_cutoffs_for_thresholds=use_profile_GA_gathering_cutoffs_for_thresholds,
                                                             use_profile_NC_noise_cutoffs_for_thresholds=use_profile_NC_noise_cutoffs_for_thresholds,
                                                             use_profile_TC_trusted_cutoffs_for_thresholds=use_profile_TC_trusted_cutoffs_for_thresholds,
                                                             turn_off_all_heruristics=turn_off_all_heruristics,
                                                             turn_off_bias_filter=turn_off_bias_filter,
                                                             MSV_threshold=MSV_threshold, Vit_threshold=Vit_threshold,
                                                             Fwd_threshold=Fwd_threshold,
                                                             turn_off_biased_composition_score_corrections=turn_off_biased_composition_score_corrections)
        common_options += " --cpu %i" % cpus_per_task
        common_options += " --qformat %s" if input_format else ""

        if handling_mode == "local":
            number_of_files = num_of_seqs_per_scan if num_of_seqs_per_scan else 5 * threads if threads else 5 * self.threads
            self.split_fasta(seqfile, splited_fasta_dir, num_of_files=number_of_files, output_prefix=output_prefix)
            input_list_of_files = sorted(os.listdir(splited_fasta_dir))
            list_of_files = []

            for filename in input_list_of_files:
                filename_prefix = self.split_filename(filename)[1]

                input_file = "%s%s" % (splited_fasta_dir, filename)
                output_file = "%s%s.hits" % (splited_output_dir, filename_prefix)
                tblout_file = "%s%s.tblout" % (splited_tblout_dir, filename_prefix) if splited_tblout_dir else None
                domtblout_file = "%s%s.domtblout" % (splited_domtblout_dir, filename_prefix) if splited_domtblout_dir else None
                pfamtblout_file = "%s%s.pfamtblout" % (splited_pfamtblout_dir, filename_prefix) if splited_pfamtblout_dir else None

                list_of_files.append((input_file, output_file, tblout_file, domtblout_file, pfamtblout_file))

            options_list = []
            out_files = []
            tblout_files = []
            domtblout_files = []
            pfamtblout_files = []

            for in_file, out_filename, tblout_file, domtblout_file, pfamtblout_file in list_of_files:
                options = common_options

                options += " --tblout %s" % tblout_file if tblout_file else ""
                options += " --domtblout %s" % domtblout_file if domtblout_file else ""
                options += " --pfamtblout %s" % pfamtblout_file if pfamtblout_file else ""
                options += " -o %s" % out_filename

                options += " %s" % hmmfile
                options += " %s" % in_file

                options_list.append(options)
                out_files.append(out_filename)
                tblout_files.append(tblout_file)
                domtblout_files.append(domtblout_file)
                pfamtblout_files.append(pfamtblout_file)

            self.parallel_execute(options_list, cmd="hmmscan", threads=threads, async_run=async_run,
                                  external_process_pool=external_process_pool)

            if combine_output_to_single_file:
                if biopython_165_compartibility:
                    CGAS.cgas(out_files, sed_string="s/^Description:.*/Description: <unknown description>/",
                              output="%s/%s.hits" % (output_dir, output_prefix))
                else:
                    CGAS.cat(out_files, output="%s/%s.hits" % (output_dir, output_prefix))

                CGAS.cat(tblout_files, output="%s/%s.tblout" % (output_dir, output_prefix))
                CGAS.cat(domtblout_files, output="%s/%s.domtblout" % (output_dir, output_prefix))
                CGAS.cat(pfamtblout_files, output="%s/%s.pfamtblout" % (output_dir, output_prefix))

            if extract_top_hits:
                for suffix in ".top_hits", ".top_hits.ids", ".not_significant.ids", ".not_found.ids":
                    os.system("cat %s/*%s > %s/%s%s\n" % (splited_hit_info, suffix, output_dir, output_prefix, suffix))

            if remove_tmp_dirs:
                for tmp_dir in directory_list:
                    shutil.rmtree(tmp_dir)

        elif handling_mode == "slurm":
            number_of_files = self.split_fasta(seqfile, splited_fasta_dir, num_of_files=threads if threads else self.threads,
                                               output_prefix=output_prefix)

            slurm_cmd_options = "hmmscan %s" % common_options

            slurm_cmd_options += " --tblout %s/%s_${SLURM_ARRAY_TASK_ID}.tblout" % (splited_tblout_dir, output_prefix) if splited_tblout_dir else ""
            slurm_cmd_options += " --domtblout %s/%s_${SLURM_ARRAY_TASK_ID}.domtblout" % (splited_domtblout_dir, output_prefix) if splited_domtblout_dir else ""
            slurm_cmd_options += " --pfamtblout %s/%s_${SLURM_ARRAY_TASK_ID}.pfamtblout" % (splited_pfamtblout_dir, output_prefix) if splited_pfamtblout_dir else ""
            slurm_cmd_options += " -o %s/%s_${SLURM_ARRAY_TASK_ID}.hits" % (splited_output_dir, output_prefix)

            slurm_cmd_options += " %s" % hmmfile
            slurm_cmd_options += " %s/%s_${SLURM_ARRAY_TASK_ID}.fasta" % (splited_fasta_dir, output_prefix)
            slurm_cmd_options += "\n\n"

            if extract_top_hits or get_clusters_from_top_hits:
                slurm_cmd_options += "%shmmer3/extract_top_hits.py " % self.check_dir_path(MAVR_scripts_dir)
                slurm_cmd_options += " -i %s/%s_${SLURM_ARRAY_TASK_ID}.hits" % (splited_output_dir, output_prefix)
                slurm_cmd_options += " -p %s/%s_${SLURM_ARRAY_TASK_ID}" % (splited_hit_info, output_prefix)
                slurm_cmd_options += " -a %s" % hmm_hit_parsing_mode

                slurm_cmd_options += "\n\n"

            #print number_of_files

            last_job_id = self.slurm_run_job(job_name,
                                             log_prefix,
                                             slurm_cmd_options,
                                             error_log_prefix,
                                             "%s%s.slurm" % (output_dir, output_prefix),
                                             task_index_list=None,
                                             start_task_index=1,
                                             end_task_index=number_of_files,
                                             max_running_jobs=max_running_jobs,
                                             max_running_time=max_running_time,
                                             cpus_per_task=cpus_per_task,
                                             max_memmory_per_cpu=max_memmory_per_cpu,
                                             modules_list=modules_list,
                                             environment_variables_dict=environment_variables_dict)
            print("Submitted job  %s" % last_job_id)

            if combine_output_to_single_file:
                if biopython_165_compartibility:
                    slurm_merging_cmd = "cat %s/* | sed 's/^Description:.*/Description: <unknown description>/' > %s/%s.hits\n" % (splited_output_dir, output_dir, output_prefix)
                else:
                    slurm_merging_cmd = "cat %s/* > %s/%s.hits\n" % (splited_output_dir, output_dir, output_prefix)
                slurm_merging_cmd += "cat %s/* > %s/%s.tblout\n" % (splited_tblout_dir, output_dir, output_prefix)
                slurm_merging_cmd += "cat %s/* > %s/%s.domtblout\n" % (splited_domtblout_dir, output_dir, output_prefix)
                slurm_merging_cmd += "cat %s/* > %s/%s.pfamtblout\n" % (splited_pfamtblout_dir, output_dir, output_prefix)

                if extract_top_hits or get_clusters_from_top_hits:
                    for suffix in ".top_hits", ".top_hits.ids", ".not_significant.ids", ".not_found.ids":
                        slurm_merging_cmd += "cat %s/*%s > %s/%s%s\n" % (splited_hit_info, suffix, output_dir, output_prefix, suffix)

                    if get_clusters_from_top_hits:
                        slurm_merging_cmd += "%shmmer3/get_clusters_from_top_hits.py " % self.check_dir_path(MAVR_scripts_dir)
                        slurm_merging_cmd += " -i %s/%s.top_hits" % (output_dir, output_prefix)
                        slurm_merging_cmd += " -o %s/%s.fam\n" % (output_dir, output_prefix)

                last_job_id = self.slurm_run_job_array("merge_%s" % job_name,
                                                       log_prefix + "_merge",
                                                       slurm_merging_cmd,
                                                       error_log_prefix + "_merge",
                                                       "%s%s_merge.slurm" % (output_dir, output_prefix),
                                                       task_index_list=None,
                                                       start_task_index=1,
                                                       end_task_index=1,
                                                       max_running_jobs=1,
                                                       max_running_time=max_running_time,
                                                       cpus_per_task=1,
                                                       max_memmory_per_cpu=max_memmory_per_cpu,
                                                       modules_list=modules_list,
                                                       environment_variables_dict=environment_variables_dict,
                                                       afterok_job_id_list=last_job_id)
                print("Submitted job  %s" % last_job_id)




            """
            self.generate_slurm_job_array_script(job_name,
                                                 log_prefix,
                                                 slurm_cmd_options,
                                                 error_log_prefix,
                                                 job_array_script_file=job_array_script_file,
                                                 task_index_list=None,
                                                 start_task_index=1,
                                                 end_task_index=number_of_files,
                                                 max_running_jobs=max_running_jobs,
                                                 max_running_time=max_running_time,
                                                 max_memmory_per_cpu=max_memmory_per_cpu)

            job_array_id = self.slurm_run_job_array(job_array_script_file)

            return job_array_id
            """

            return last_job_id

    def hmmsearch(self, hmmfile, seqfile, outfile, multialignout=None, tblout=None, domtblout=None, pfamtblout=None,
                  dont_output_alignments=False, model_evalue_threshold=None, model_score_threshold=None,
                  domain_evalue_threshold=None, domain_score_threshold=None,
                  model_evalue_significant_threshold=None, model_score_significant_threshold=None,
                  domain_evalue_significant_threshold=None, domain_score_significant_threshold=None,
                  use_profile_GA_gathering_cutoffs_for_thresholds=False,
                  use_profile_NC_noise_cutoffs_for_thresholds=False,
                  use_profile_TC_trusted_cutoffs_for_thresholds=False,
                  turn_off_all_heruristics=False, turn_off_bias_filter=False,
                  MSV_threshold=None, Vit_threshold=None, Fwd_threshold=None,
                  turn_off_biased_composition_score_corrections=None,
                  input_format=None):

        options = self.__parse_hmmsxxx_common_options(tblout=tblout, domtblout=domtblout,
                                                      pfamtblout=pfamtblout,
                                                      dont_output_alignments=dont_output_alignments,
                                                      model_evalue_threshold=model_evalue_threshold,
                                                      model_score_threshold=model_score_threshold,
                                                      domain_evalue_threshold=domain_evalue_threshold,
                                                      domain_score_threshold=domain_score_threshold,
                                                      model_evalue_significant_threshold=model_evalue_significant_threshold,
                                                      model_score_significant_threshold=model_score_significant_threshold,
                                                      domain_evalue_significant_threshold=domain_evalue_significant_threshold,
                                                      domain_score_significant_threshold=domain_score_significant_threshold,
                                                      use_profile_GA_gathering_cutoffs_for_thresholds=use_profile_GA_gathering_cutoffs_for_thresholds,
                                                      use_profile_NC_noise_cutoffs_for_thresholds=use_profile_NC_noise_cutoffs_for_thresholds,
                                                      use_profile_TC_trusted_cutoffs_for_thresholds=use_profile_TC_trusted_cutoffs_for_thresholds,
                                                      turn_off_all_heruristics=turn_off_all_heruristics,
                                                      turn_off_bias_filter=turn_off_bias_filter,
                                                      MSV_threshold=MSV_threshold, Vit_threshold=Vit_threshold,
                                                      Fwd_threshold=Fwd_threshold,
                                                      turn_off_biased_composition_score_corrections=turn_off_biased_composition_score_corrections)
        options += " --cpu %i" % self.threads
        options += " -o %s" % outfile
        options += " -A %s" % multialignout
        options += " --tformat %s" if input_format else ""
        options += " %s" % hmmfile
        options += " %s" % seqfile

        self.execute(options, cmd="hmmsearch")

    @staticmethod
    def extract_dom_ids_hits_from_domtblout(domtblout_file, output_file=None):
        hits_dict = SynDict()
        hits_dict.read(domtblout_file, header=False, separator=None, allow_repeats_of_key=True,
                       key_index=3, value_index=1, comments_prefix="#")
        if output_file:
            hits_dict.write(output_file, splited_values=True)
        return hits_dict

    @staticmethod
    def extract_dom_names_hits_from_domtblout(domtblout_file, output_file):
        hits_dict = SynDict()
        hits_dict.read(domtblout_file, header=False, separator=None, allow_repeats_of_key=True,
                       key_index=3, value_index=0, comments_prefix="#")
        if output_file:
            hits_dict.write(output_file, splited_values=True)
        return hits_dict

    def extract_query_ids_with_hits(self, domtblout_file, output_file):
        return self.extract_ids_from_file(domtblout_file, output_file=output_file, header=False, column_separator=" ",
                                          comments_prefix="#", column_number=0)

    def extract_top_hits(self, hmmer_hits, output_prefix, parsing_mode="index_db"): #top_hits_file, top_hits_ids_file=None,not_significant_ids_file=None, not_found_ids_file=None):

        top_hits_ids = IdList()
        not_significant_ids = IdList()
        not_found_ids = IdList()

        top_hits_file = "%s.top_hits" % output_prefix
        top_hits_ids_file = "%s.top_hits.ids" % output_prefix
        not_significant_ids_file = "%s.not_significant.ids" % output_prefix
        not_found_ids_file = "%s.not_found.ids" % output_prefix

        index_file = "%s.hmmer_hits.tmp.idx" % output_prefix

        #hmm_dict = SearchIO.index_db(index_file, hmmer_hits, "hmmer3-text")

        hmm_dict = self.parse_search_file(hmmer_hits, parsing_mode, format="hmmer3-text", index_file=None)

        with open(top_hits_file, "w") as out_fd:
            out_fd.write("#query\thit\tevalue\tbitscore\n")

            for query in hmm_dict:
                if hmm_dict[query].hits:
                    if hmm_dict[query][0].is_included:
                        out_fd.write("%s\t%s\t%s\t%s\n" % (query, hmm_dict[query][0].id, hmm_dict[query][0].evalue,
                                                           hmm_dict[query][0].bitscore))
                        top_hits_ids.append(query)
                    else:
                        not_significant_ids.append(query)
                else:
                    not_found_ids.append(query)
        if parsing_mode == "index_db":
            os.remove(index_file)

        for id_list, id_file in zip([not_significant_ids, not_found_ids, top_hits_ids],
                                    [not_significant_ids_file, not_found_ids_file, top_hits_ids_file]):
            id_list.write(id_file)


    @staticmethod
    def get_families_from_top_hits(top_hits_file, fam_file):

        hit_dict = SynDict()
        hit_dict.read(top_hits_file, allow_repeats_of_key=True, key_index=1, value_index=0, comments_prefix="#")
        hit_dict.write(fam_file, splited_values=True)

        return hit_dict


if __name__ == "__main__":
    pass



#!/usr/bin/env python
__author__ = 'mahajrod'

import numpy as np
import pandas as pd


class HGMDvariants(pd.DataFrame):
    def __init__(self, input, separator="\t", header=None):

        pd.DataFrame.__init__(self)

        if isinstance(input, str) or isinstance(input, file):
            pd.DataFrame.__init__(self, pd.read_csv(input, sep=separator, header=0 if header is True else header))
        elif input:
            pd.DataFrame.__init__(self, input)
        self.disease_column_name = "disease"
        self.gene_column_name = "gene"
        self.hgvs_column_name = "hgvsall"
        self.codon_change_column = "base"
        self.aa_change_column = "amino"
        self.changed_codon_position = "codon"

        self.hgvs_snp_template1 = '(\d+)([A-Za-z])to([A-Za-z])'
        self.hgvs_snp_template2 = '([A-Za-z])(\d+)([A-Za-z*])'

        self.hgvs_indel_template1 = '(-?\d+[\-\+]?\d*)_?(-?\d+[\-\+]?\d*)?(del|dup|ins)([A-Za-z]+)'
        self.hgvs_indel_template2 = '(-?\d+[\-\+]?\d*)_?(-?\d+[\-\+]?\d*)?(del|dup|ins)(\d+)'
        self.hgvs_indel_template3 = '(-?\d+[\-\+]?\d*)_?(-?\d+[\-\+]?\d*)?(del|dup|ins)'

        self.hgvs_hgmd_indel_template1 = '(-?\d+[\-\+]?\d*)_?(-?\d+[\-\+]?\d*)?(del)([A-Za-z]+)(ins)([A-Za-z]+)?'
        self.hgvs_hgmd_indel_template2 = '(-?\d+[\-\+]?\d*)_?(-?\d+[\-\+]?\d*)?(del)(\d+)(ins)(\d+)?'
        self.hgvs_hgmd_indel_template3 = '(-?\d+[\-\+]?\d*)_?(-?\d+[\-\+]?\d*)?(delins)'

    def get_snp_hgvs_dataframes(self):
        tmp = self[self.hgvs_column_name].str.split("|", expand=True)
        return tmp[0].str.extract(self.hgvs_snp_template1), \
               tmp[1].str.extract(self.hgvs_snp_template2)

    def get_codon_change_dataframe(self):
        return self[self.codon_change_column].str.split("-", expand=True)

    def get_aa_change_dataframe(self):
        return self[self.aa_change_column].str.split("-", expand=True)

    def get_changed_codon_position_dataframe(self):
        return self[self.changed_codon_position]

    def get_indel_hgvs_dataframes(self):
        tmp = self[self.hgvs_column_name].str.split("|", expand=True)
        return tmp[0].str.extract(self.hgvs_indel_template1), \
               tmp[1].str.extract(self.hgvs_indel_template2), \
               tmp[2].str.extract(self.hgvs_indel_template3)

    def get_hgmd_indel_hgvs_dataframes(self):
        tmp = self[self.hgvs_column_name].str.split("|", expand=True)
        return tmp[0].str.extract(self.hgvs_hgmd_indel_template1), \
               tmp[1].str.extract(self.hgvs_hgmd_indel_template2), \
               tmp[2].str.extract(self.hgvs_hgmd_indel_template3)

    def extract_snp_coordinates(self):
        hgvs_nuc, hgvs_aa = self.get_snp_hgvs_dataframes()

        results_df = pd.concat([self[self.disease_column_name],               # 1 column
                                self[self.gene_column_name],                  # 1 column
                                hgvs_nuc[0],                                  # 1 column
                                hgvs_nuc[1],                                  # 1 column
                                hgvs_nuc[2],                                  # 1 column
                                self.get_codon_change_dataframe(),            # 2 columns
                                self.get_changed_codon_position_dataframe(),  # 1 column
                                hgvs_aa[0],                                   # 1 column
                                hgvs_aa[2],                                   # 1 column
                                self.get_aa_change_dataframe()                # 2 columns
                                ],
                               axis=1
                               )

        results_df.columns = ["disease",
                              "gene",
                              "nuc_pos",
                              "ref_allel",
                              "alt_allel",
                              "ref_codon",
                              "alt_codon",
                              "codon_pos",
                              "ref_aa,1l",
                              "alt_aa,1l",
                              "ref_aa,3l",
                              "alt_aa,3l"]

        return results_df

    def extract_indel_coordinates(self):
        hgvs_indel_1, hgvs_indel_2, hgvs_indel_3 = self.get_indel_hgvs_dataframes()

        results_df = pd.concat([self[self.disease_column_name],                 # 1 column
                                self[self.gene_column_name],                    # 1 column
                                hgvs_indel_1[2],                                # 1 column
                                hgvs_indel_1[0],                                # 1 column
                                hgvs_indel_1[1],                                # 1 column
                                hgvs_indel_1[3],                                # 1 column
                                hgvs_indel_2[3],                                # 1 column
                                self.get_changed_codon_position_dataframe(),    # 1 column
                                ],
                               axis=1
                               )

        results_df.columns = ["disease",
                              "gene",
                              "indel_type",
                              "indel_start",
                              "indel_end",
                              "indel_sequence",
                              "indel_length",
                              "codon_pos"]

        return results_df

    def extract_hgmd_indel_coordinates(self):
        hgvs_hgmd_indel_1, hgvs_hgmd_indel_2, hgvs_hgmd_indel_3 = self.get_hgmd_indel_hgvs_dataframes()

        results_df = pd.concat([self[self.disease_column_name],                 # 1 column
                                self[self.gene_column_name],                    # 1 column
                                hgvs_hgmd_indel_3[2],                           # 1 column
                                hgvs_hgmd_indel_1[0],                           # 1 column
                                hgvs_hgmd_indel_1[1],                           # 1 column
                                hgvs_hgmd_indel_1[3],                           # 1 column
                                hgvs_hgmd_indel_2[3],                           # 1 column
                                hgvs_hgmd_indel_1[5],                           # 1 column
                                hgvs_hgmd_indel_2[5],                           # 1 column
                                self.get_changed_codon_position_dataframe(),    # 1 column
                                ],
                               axis=1
                               )

        results_df.columns = ["disease",
                              "gene",
                              "indel_type",
                              "indel_start",
                              "indel_end",
                              "del_sequence",
                              "del_length",
                              "ins_sequence",
                              "ins_length",
                              "codon_pos"]

        return results_df

    def extract_variant_coordinates(self, variant_type):
        if variant_type == "snp":
            return self.extract_snp_coordinates()
        elif variant_type == "indel":
            return self.extract_indel_coordinates()
        elif variant_type == "hgmd_indel":
            return self.extract_hgmd_indel_coordinates()
        else:
            raise ValueError("ERROR!!! Unknown or not implemented for coordinate extraction variant type")


__author__ = 'mahajrod'

import os
import sys
from setuptools import setup, find_packages

dependencies = ['scipy', 'numpy', 'pandas', 'matplotlib', 'matplotlib-venn',
                'biopython', 'xmltodict', 'bcbio-gff', 'statsmodels']
if sys.version_info[0] == 3:
    dependencies += ["ete3"]
elif sys.version_info[0] == 2:
    dependencies += ["ete2"]
else:
    raise ValueError("ERROR!!! Unsupported python version: %s" % str(sys.version_info[0]))

setup(name='RouToolPa',
      version='0.83',
      packages=find_packages(),
      author='Sergei F. Kliver',
      author_email='mahajrod@gmail.com',
      install_requires=dependencies,
      long_description=open(os.path.join(os.path.dirname(__file__), 'README.md')).read(),)



#!/usr/bin/env python
__author__ = 'mahajrod'

from RouToolPa.Tools.Abstract import Tool
from RouToolPa.Tools.Abstract import JavaTool


class RealignerTargetCreator(JavaTool):
    def __init__(self,  java_path="", max_threads=4, jar_path="", max_memory=None, timelog=None):
        JavaTool.__init__(self, "GenomeAnalysisTK.jar -T RealignerTargetCreator", java_path=java_path,
                          max_threads=max_threads, jar_path=jar_path, max_memory=max_memory,
                          timelog=timelog)

    def create(self, reference, alignment, output="forIndelRealigner.intervals", known_indels_vcf=None,
               max_interval_size=None, min_reads_cov=None, mismatch_fraction=None, window_size=None,
               default_base_qualities=None):

        options = " -nt %i" % self.threads
        options += " -R %s" % reference
        options += " -I %s" % alignment
        options += " -o %s" % output
        options += " --known %s" % known_indels_vcf if known_indels_vcf else ""
        options += " -maxInterval %i" % max_interval_size if max_interval_size else ""
        options += " -minReads %i" % min_reads_cov if min_reads_cov else ""
        options += " -mismatch %i" % mismatch_fraction if mismatch_fraction else ""
        options += " -window %i" % window_size if window_size else ""
        options += " --defaultBaseQualities %i" % default_base_qualities if default_base_qualities else ""

        self.execute(options)

if __name__ == "__main__":
    import os
    workdir = "/media/mahajrod/d9e6e5ee-1bf7-4dba-934e-3f898d9611c8/Data/Alsu/fastq/001/alignment_tmap_Alsu24mc"
    os.chdir(workdir)
    #gatk_dir =
    reference = "/media/mahajrod/d9e6e5ee-1bf7-4dba-934e-3f898d9611c8/Data/Alsu/reference/Alsu24mc/Alsu24mc.fasta"
    alignment = "/media/mahajrod/d9e6e5ee-1bf7-4dba-934e-3f898d9611c8/Data/Alsu/fastq/001/alignment_tmap_Alsu24mc/001_trimmed_sorted_rm_pcr_chrom.bam"
    RealignerTargetCreator = RealignerTargetCreator(jar_path="/home/mahajrod/Repositories/genetic/NGS_tools/GenomeAnalysisTK-3.2-0")
    RealignerTargetCreator.create(reference, alignment)


__author__ = 'mahajrod'

from RouToolPa.Tools.GATK.PrintReads import PrintReads
from RouToolPa.Tools.GATK.CatVariants import CatVariants
from RouToolPa.Tools.GATK.GenotypeGVCFs import GenotypeGVCFs
from RouToolPa.Tools.GATK.IndelRealigner import IndelRealigner
from RouToolPa.Tools.GATK.SelectVariants import SelectVariants
from RouToolPa.Tools.GATK.CombineVariants import CombineVariants
from RouToolPa.Tools.GATK.HaplotypeCaller import HaplotypeCaller
from RouToolPa.Tools.GATK.ValidateVariants import ValidateVariants
from RouToolPa.Tools.GATK.BaseRecalibrator import BaseRecalibrator
from RouToolPa.Tools.GATK.UnifiedGenotyper import UnifiedGenotyper
from RouToolPa.Tools.GATK.AnalyzeCovariates import AnalyzeCovariates
from RouToolPa.Tools.GATK.VariantFiltration import VariantFiltration
from RouToolPa.Tools.GATK.ApplyRecalibration import ApplyRecalibration
from RouToolPa.Tools.GATK.VariantRecalibrator import VariantRecalibrator
from RouToolPa.Tools.GATK.RealignerTargetCreator import RealignerTargetCreator
from RouToolPa.Tools.GATK.FastaAlternateReferenceMaker import FastaAlternateReferenceMaker


PrintReads = PrintReads()
CatVariants = CatVariants()
GenotypeGVCFs = GenotypeGVCFs()
IndelRealigner = IndelRealigner()
SelectVariants = SelectVariants()
CombineVariants = CombineVariants()
HaplotypeCaller = HaplotypeCaller()
ValidateVariants = ValidateVariants()
BaseRecalibrator = BaseRecalibrator()
UnifiedGenotyper = UnifiedGenotyper()
AnalyzeCovariates = AnalyzeCovariates()
VariantFiltration = VariantFiltration()
ApplyRecalibration = ApplyRecalibration()
VariantRecalibrator = VariantRecalibrator()
RealignerTargetCreator = RealignerTargetCreator()
FastaAlternateReferenceMaker = FastaAlternateReferenceMaker()



#!/usr/bin/env python

from collections import OrderedDict

LAST_TAB_COLS = OrderedDict({
                             "score": 0,
                             "target_id":   1,
                             "target_start": 2,
                             "target_hit_len": 3,
                             "target_strand": 4,
                             "target_len": 5,
                             "query_id": 6,
                             "query_start": 7,
                             "query_hit_len": 8,
                             "query_strand": 9,
                             "query_len": 10,
                             "alignment": 11,
                             "EG2": 12,
                             "E": 13,
                              })



__author__ = 'mahajrod'
import os
from copy import deepcopy
from collections import OrderedDict
import numpy as np
from BCBio import GFF
from Bio import SeqIO
from Bio.SeqFeature import SeqFeature, FeatureLocation
from RouToolPa.Routines.Sequence import SequenceRoutines
from RouToolPa.Collections.General import IdSet, SynDict, IdList


class AnnotationsRoutines(SequenceRoutines):
    def __init__(self):
        SequenceRoutines.__init__(self)

        self.GFF_SCAFFOLD_COLUMN = 0
        self.GFF_SOURCE_COLUMN = 1
        self.GFF_FEATURETYPE_COLUMN = 2
        self.GFF_START_COLUMN = 3
        self.GFF_END_COLUMN = 4
        self.GFF_SCORE_COLUMN = 5
        self.GFF_STRAND_COLUMN = 6
        self.GFF_PHASE_COLUMN = 7
        self.GFF_ATTRIBUTE_COLUMN = 8

        self.BED_SCAFFOLD_COLUMN = 0
        self.BED_START_COLUMN = 1
        self.BED_END_COLUMN = 2

    @staticmethod
    def record_with_extracted_annotations_generator(gff_file,
                                                    annotation_ids,
                                                    white_list_of_annotation_types):
        for record in GFF.parse(open(gff_file)):
            new_record = deepcopy(record)
            new_record.features = []
            for feature in record.features:
                if (feature.id in annotation_ids) and (feature.type in white_list_of_annotation_types):
                    new_record.features.append(feature)
            if len(new_record.features) > 0:
                yield new_record

    def extract_annotation_from_gff(self, input_gff, annotation_ids,
                                    white_list_of_annotation_types, output_gff):

        GFF.write(self.record_with_extracted_annotations_generator(input_gff, annotation_ids,
                                                                   white_list_of_annotation_types),
                  open(output_gff, "w"))

    @staticmethod
    def record_with_extracted_transcripts_generator(gff_file,
                                                    transcript_ids):
        for record in GFF.parse(open(gff_file)):
            new_record = deepcopy(record)
            new_record.features = []
            for feature in record.features:
                if (feature.type == "mRNA" or feature.type == "transcript") and (feature.id in transcript_ids):
                    new_record.features.append(feature)
                elif feature.type == "gene":
                    new_feature = deepcopy(feature)
                    new_feature.sub_features = []

                    for subfeature in feature.sub_features:
                        if (subfeature.type == "mRNA" or subfeature.type == "transcript") and (subfeature.id in transcript_ids):
                            new_feature.sub_features.append(subfeature)
                    if len(new_feature.sub_features) > 0:
                        new_record.features.append(new_feature)
            if len(new_record.features) > 0:
                yield new_record

    def extract_transcripts_by_ids(self, input_gff, transcript_id_file, output_gff):
        transcript_ids = IdSet()
        transcript_ids.read(transcript_id_file, header=False)
        GFF.write(self.record_with_extracted_transcripts_generator(input_gff, transcript_ids),
                  open(output_gff, "w"))

    @staticmethod
    def replace_region_names_in_gff(input_gff, synonyms_file, output_gff):
        syn_dict = SynDict()
        syn_dict.read(synonyms_file, comments_prefix="#")
        with open(input_gff, "r") as in_fd:
            with open(output_gff, "w") as out_fd:
                for line in in_fd:
                    if line[0] == "#":
                        out_fd.write(line)
                    else:
                        line_list = line.split("\t")
                        if line_list[0] in syn_dict:
                            line_list[0] = syn_dict[line_list[0]]
                            out_fd.write("\t".join(line_list))
                        else:
                            out_fd.write(line)

    @staticmethod
    def get_transcript_to_pep_accordance_from_gtf(gtf_file, output_file, comment_symbol="#"):
        """
        Tested on gtf files from Ensembl relealese 70
        """
        accordance_dict = SynDict()
        with open(gtf_file, "r") as gtf_fd:
            for line in gtf_fd:
                if line[0] == comment_symbol:
                    continue
                tmp_list = line.strip().split("\t")
                tmp_list = tmp_list[-1].split(";")
                protein_id = None
                transcript_id = None
                #print tmp_list
                for entry in tmp_list:
                    tmp_entry = entry.split()

                    if len(tmp_entry) != 2:
                        continue
                    if tmp_entry[0] == "transcript_id":
                        #print "tttt"
                        transcript_id = tmp_entry[1][1:-1]  # remove quotes
                    elif tmp_entry[0] == "protein_id":
                        #print "ppppp"
                        protein_id = tmp_entry[1][1:-1]

                if (transcript_id is not None) and (protein_id is not None):
                    if transcript_id in accordance_dict:
                        accordance_dict[transcript_id].add(protein_id)
                    else:
                        accordance_dict[transcript_id] = {protein_id}
        accordance_dict.write(output_file, splited_values=True)

    @staticmethod
    def fix_gff_coordinates_order(input_gff, output_gff):
        fixed_lines_number = 0
        malformed_lines_number = 0
        with open(input_gff, "r") as in_fd:
            with open(output_gff, "w") as out_fd:
                for line in in_fd:
                    if line[0] == "#":
                        out_fd.write(line)
                        continue
                    tmp = line.split("\t")

                    try:
                        start = int(tmp[3])
                        end = int(tmp[4])
                    except ValueError:
                        malformed_lines_number += 1
                        continue

                    if start <= end:
                        out_fd.write(line)
                        continue

                    tmp[3], tmp[4] = tmp[4], tmp[3]
                    out_fd.write("\t".join(tmp))

                    fixed_lines_number += 1

        print("Fixed lines: %i" % fixed_lines_number)
        print("Malformed lines(removed): %i" % malformed_lines_number)

    @staticmethod
    def fix_absent_feature_type_field(input_gff, output_gff, feature_type):
        fixed_lines_number = 0
        malformed_lines_number = 0
        malformed_lines_list = []
        line_number = 0
        with open(input_gff, "r") as in_fd:
            with open(output_gff, "w") as out_fd:
                for line in in_fd:
                    line_number += 1
                    if line[0] == "#":
                        out_fd.write(line)
                        continue
                    tmp = line.split("\t")

                    try:
                        start = int(tmp[3])
                        end = int(tmp[4])

                    except ValueError:
                        malformed_lines_number += 1
                        malformed_lines_list.append(line_number)
                        tmp = tmp[:2] + [feature_type] + tmp[2:]
                        out_fd.write("\t".join(tmp))
                        continue

                    out_fd.write(line)

    @staticmethod
    def add_alias_to_feature(input_gff, output_gff, syn_file, feature_type_list=(), name_field_list=("ID",),
                             alias_field="Alias", key_column=0, value_column=1):
        syn_dict = SynDict(filename=syn_file, comments_prefix="#", split_values=True,
                           values_separator=",", key_index=key_column, value_index=value_column)
        with open(input_gff, "r") as in_fd:
            with open(output_gff, "w") as out_fd:
                for line in in_fd:

                    if line[0] == "#":
                        out_fd.write(line)
                        continue
                    tmp = line.strip().split("\t")
                    if feature_type_list:  # skip feature types not present in feature type list
                        if tmp[2] not in feature_type_list:
                            out_fd.write(line)
                            continue
                    description_list = tmp[8].split(";")

                    feature_aliases = []
                    for description_entry in description_list:
                        entry_list = description_entry.split("=")
                        if entry_list[0] not in name_field_list:
                            continue
                        feature_aliases += entry_list[1].split(",")

                    if not feature_aliases:
                        print("No ID was found for this line: \n%s" % line)

                    feature_synonyms_list = []
                    syn_lines = 0
                    for name in feature_aliases:
                        if name in syn_dict:
                            feature_synonyms_list += syn_dict[name]
                            syn_lines += 1
                    #print syn_dict
                    if syn_lines > 1:
                        print("Warning!!!. For gene with names %s were found more 1 synonym group." % ",".join(feature_aliases))

                    for i in range(0, len(description_list)):
                        entry_list = description_list[i].split("=")
                        if entry_list[0] != alias_field:
                            continue

                        alias_list = set(entry_list[1].split(",") + feature_synonyms_list)
                        description_list[i] = "%s=%s" % (entry_list[0], ",".join(alias_list))
                    else:
                        description_list.append("%s=%s" % (alias_field, ",".join(feature_synonyms_list)))

                    out_fd.write("\t".join(tmp[:8] + [";".join(description_list)]) + "\n")

    @staticmethod
    def count_total_feature_length_from_gff(input_gff, output_prefix, features_to_count=None):
        len_file = "%s.len" % output_prefix
        stat_file = "%s.stat" % output_prefix

        total_feature_length = 0
        feature_number = 0

        with open(input_gff, "r") as in_fd:
            with open(len_file, "w") as len_fd:
                for line in in_fd:
                    if line[0] == "#":
                        continue
                    tmp = line.split("\t")
                    feature = tmp[2]
                    if features_to_count is not None:
                        if feature not in features_to_count:
                            continue

                    start = int(tmp[3])
                    end = int(tmp[4])

                    feature_number += 1
                    feature_length = end - start + 1

                    len_fd.write("%i\n" % feature_length)

                    total_feature_length += feature_length

        stat_string = "Features\t%s\n" % (",".join(features_to_count) if features_to_count else "all")
        stat_string += "Number of features\t%i\n" % feature_number
        stat_string += "Total length\t%i\n" % total_feature_length

        print(stat_string)
        with open(stat_file, "w") as stat_fd:
            stat_fd.write(stat_string)

    def get_feature_length_distribution_from_gff(self, input_gff, output_prefix, feature_list=None):
        from RouToolPa.Routines import MatplotlibRoutines

        len_file = "%s.len" % output_prefix
        stat_file = "%s.stat" % output_prefix

        feature_length_list = []
        total_feature_length = 0
        feature_number = 0
        feature_type_set = set()

        with open(input_gff, "r") as in_fd:
            for line in in_fd:
                if line[0] == "#":
                    continue
                tmp = line.split("\t")
                feature = tmp[self.GFF_FEATURETYPE_COLUMN]
                feature_type_set.add(feature)
                if feature_list is not None:
                    if isinstance(feature_list, str):
                        if feature != feature_list:
                            continue
                    else:
                        if feature not in feature_list:
                            continue

                start = int(tmp[self.GFF_START_COLUMN])
                end = int(tmp[self.GFF_END_COLUMN])

                feature_number += 1
                feature_length = end - start + 1
                feature_length_list.append(feature_length)
                #len_fd.write("%i\n" % feature_length)

                total_feature_length += feature_length

        stat_string = "Features\t%s\n" % (",".join(feature_list) if feature_list else "all")
        stat_string += "Number of features\t%i\n" % feature_number
        stat_string += "Total length\t%i\n" % total_feature_length

        print(stat_string)
        with open(stat_file, "w") as stat_fd:
            stat_fd.write(stat_string)

        feature_length_list = np.array(feature_length_list)
        np.savetxt(len_file, feature_length_list, fmt='%i')

        feature_name = "feature"

        if len(feature_type_set) == 1:
            feature_name = list(feature_type_set)[0]
        elif feature_list is None:
            feature_name = "feature"
        elif isinstance(feature_list, str):
            feature_name = feature_list
        elif len(feature_list) == 1:
            feature_name = feature_list[0]
        else:
            feature_name = "feature"

        MatplotlibRoutines.draw_histogram(feature_length_list, output_prefix=output_prefix + ".all",
                                          xlabel="Feature length", ylabel="N of features",
                                          title="Distribution of %s lengths" % feature_name,
                                          ylogbase=10, xlogbase=10,
                                          bins_list=[1, 10, 100, 1000, 10000, 100000, 1000000, 10000000],
                                          close_figure=True)

        MatplotlibRoutines.draw_histogram(feature_length_list, output_prefix=output_prefix + ".max_10000",
                                          width_of_bins=100, max_threshold=10000, min_threshold=1,
                                          xlabel="Feature length", ylabel="N of features",
                                          title="Distribution of %s lengths" % feature_name,
                                          close_figure=True)

        MatplotlibRoutines.draw_histogram(feature_length_list, output_prefix=output_prefix + ".max_1000",
                                          width_of_bins=10, max_threshold=1000, min_threshold=1,
                                          xlabel="Feature length", ylabel="N of features",
                                          title="Distribution of %s lengths" % feature_name,
                                          close_figure=True)

    @staticmethod
    def get_feature_to_parent_correspondence_from_gff(input_gff, output, feature_list=("mRNA",),
                                                      id_entry="ID", parental_id_entry="Parent"):
        with open(input_gff, "r") as in_fd:
            with open(output, "w") as out_fd:
                for line in in_fd:
                    if line[0] == "#":
                        continue
                    tmp = line.strip().split("\t")
                    if tmp[2] not in feature_list:
                        continue
                    tmp = map(lambda s: s.split("="), tmp[-1].split(";"))

                    feature_id = None
                    parental_id = None

                    for entry, value in tmp:
                        if entry == id_entry:
                            feature_id = value
                        elif entry == parental_id_entry:
                            parental_id = value
                    if (parental_id is None) or (feature_id is None):
                        print ("Absent feature id or parental id in following line:")
                        print (line)
                        continue
                    out_fd.write("%s\t%s\n" % (parental_id, feature_id))

    @staticmethod
    def add_length_to_accordance_file(accordance_file, length_file, output_prefix):

        accordance_dict = SynDict(filename=accordance_file, allow_repeats_of_key=True)
        length_dict = SynDict(filename=length_file, expression=int)
        print(length_dict)
        longest_list = IdList()

        all_output_file = "%s.all.correspondence" % output_prefix
        longest_output_file = "%s.longest.correspondence" % output_prefix
        longest_id_file = "%s.longest.ids" % output_prefix

        with open(all_output_file, "w") as all_out_fd:
            with open(longest_output_file, "w") as longest_out_fd:
                for gene in accordance_dict:
                    current_transcript = None
                    current_length = 0
                    for transcript in accordance_dict[gene]:
                        if length_dict[transcript] > current_length:
                            current_transcript = transcript
                            current_length = length_dict[transcript]
                        all_out_fd.write("%s\t%s\t%i\n" % (gene, transcript, length_dict[transcript]))

                    longest_out_fd.write("%s\t%s\t%i\n" % (gene, current_transcript, current_length))
                    longest_list.append(current_transcript)
        longest_list.write(longest_id_file)

    @staticmethod
    def parse_gff_annotation_string_to_dict(gff_annotation_string):

        annotation_dict = OrderedDict()
        for entry in gff_annotation_string.split(";"):
            key, value = entry.split("=")
            annotation_dict[key] = value.split(",")

        return annotation_dict

    def extract_sequences_by_gff(self, input_file, gff_file, output_file, type_list=("gene",), parsing_mode="parse",
                                 tmp_index_file="temp.idx", format="fasta"):

        annotations_dict = SeqIO.to_dict(GFF.parse(open(gff_file)))
        #print annotations_dict
        #print("Parsing %s..." % args.input)
        sequence_dict = self.parse_seq_file(input_file, parsing_mode, format, index_file=tmp_index_file)  # SeqIO.index_db(tmp_index_file, args.input_file, format=args.format)

        SeqIO.write(self.record_generator(annotations_dict, sequence_dict, type_list), output_file,
                    format=format)

        if parsing_mode == "index_db":
            os.remove(tmp_index_file)

    def extract_gff_records_by_description_value(self, input_gff, output_gff, field_id_list, value_list,
                                                retain_comments=False):
        with open(input_gff, "r") as in_fd:
            with open(output_gff, "w") as out_fd:
                for line in in_fd:
                    if line[0] == "#":
                        if retain_comments:
                            out_fd.write(line)
                        continue
                    description_dict = self.get_description_dict_from_gff_string(line, split_values=True, value_separator=",")
                    found = False
                    for field_id in field_id_list:
                        if field_id in description_dict:
                            #print "aaaaa"
                            for value in description_dict[field_id]:
                                if value in value_list:
                                    out_fd.write(line)
                                    found = True
                                    break
                            if found:
                                break

    @staticmethod
    def get_description_dict_from_gff_string(gff_string, split_values=False, value_separator=","):
        if gff_string[0] == "#":
            return None

        description_list = gff_string.strip().split("\t")[-1].split(";")
        description_dict = OrderedDict()

        for entry in description_list:
            key, value = entry.split("=")
            description_dict[key] = value.split(value_separator) if split_values else value

        return description_dict

    def filter_gff_by_description(self, input_gff, output_gff, filtered_out_gff, expression):

        with open(input_gff, "r") as in_fd:
            with open(output_gff, "w") as out_fd:
                with open(filtered_out_gff, "w") as filtered_out_fd:
                    for line in in_fd:
                        if line[0] == "#":
                            out_fd.write(line)
                            continue

                        #print line
                        #print expression(self.get_description_dict_from_gff_string(line))

                        if expression(self.get_description_dict_from_gff_string(line)):
                            out_fd.write(line)
                        else:
                            filtered_out_fd.write(line)

    def add_flanks_to_gff_record(self, input_gff, output_prefix, left_flank_len, right_flank_len, fasta_file,
                                 coords_description_entry="core_seq_coords", id_description_entry="ID"):
        sequence_length_dict = self.get_lengths_from_seq_file(fasta_file)
        shorter_flanks_dict = SynDict()

        output_gff = "%s.gff" % output_prefix
        short_flanks_file = "%s.short_flanks.dat" % output_prefix

        with open(input_gff, "r") as in_fd:
            with open(output_gff, "w") as out_fd:
                for line in in_fd:
                    if line[0] == "#":
                        out_fd.write(line)
                        continue
                    line_list = line.strip().split("\t")
                    scaffold = line_list[0]
                    start = int(line_list[3])
                    end = int(line_list[4])

                    record_id = OrderedDict(map(lambda s: s.split("="), line_list[8].split(";")))[id_description_entry]

                    line_list[8] += ";%s=%i,%i" % (coords_description_entry, start, end)

                    if line_list[6] == "-":
                        if start - right_flank_len > 0:
                            line_list[3] = str(start - right_flank_len)
                            right_flank_length = right_flank_len
                        else:
                            right_flank_length = start - 1
                            line_list[3] = "1"

                        if end + left_flank_len <= sequence_length_dict[line_list[0]]:
                            line_list[4] = str(end + left_flank_len)
                            left_flank_length = left_flank_len
                        else:
                            left_flank_length = sequence_length_dict[line_list[0]] - end
                            line_list[4] = sequence_length_dict[line_list[0]]
                    else:
                        if start - left_flank_len > 0:
                            line_list[3] = str(start - left_flank_len)
                            left_flank_length = left_flank_len
                        else:
                            left_flank_length = start - 1
                            line_list[3] = "1"

                        if end + right_flank_len <= sequence_length_dict[line_list[0]]:
                            line_list[4] = str(end + right_flank_len)
                            right_flank_length = right_flank_len
                        else:
                            right_flank_length = sequence_length_dict[line_list[0]] - end
                            line_list[4] = str(sequence_length_dict[line_list[0]])

                    if (left_flank_length < left_flank_len) or (right_flank_length < right_flank_len):
                        print("%s: Short flank" % record_id)
                        shorter_flanks_dict[record_id] = "%i,%i" % (left_flank_length, right_flank_length)
                    line_list[8] += ";%s_relative=%i,%i\n" % (coords_description_entry,
                                                              1 + (right_flank_length if line_list[6] == "-" else left_flank_length),
                                                              end - start + 1 + (right_flank_length if line_list[6] == "-" else left_flank_length))
                    """
                    print( line)
                    print( line_list)
                    for element in line_list:
                        print (element)
                        print (type(element))
                    """
                    out_fd.write("\t".join(line_list))

        shorter_flanks_dict.write(short_flanks_file)

    def convert_gff_to_bed(self, input_gff, output_bed, id_entry="ID", feature_type_list=[]):
        with open(input_gff, "r") as gff_fd:
            with open(output_bed, "w") as bed_fd:
                for line in gff_fd:
                    if line[0] == "#":
                        continue
                    tmp_list = line.strip().split("\t")
                    if feature_type_list:
                        if tmp_list[2] not in feature_type_list:
                            continue
                    description_dict = self.parse_gff_annotation_string_to_dict(tmp_list[-1])

                    bed_fd.write("%s\t%s\t%s\t%s\t%s\t%s\n" % (tmp_list[0],
                                                               tmp_list[3],
                                                               tmp_list[4],
                                                               description_dict[id_entry],
                                                               tmp_list[5],
                                                               tmp_list[6]))

    def convert_bedgraph_to_gff(self,input_bed, output_gff, feature_type, id_prefix="ID", source="source"):
        with self.metaopen(output_gff, "w") as out_fd:
            index = 1
            for line_list in self.file_line_as_list_generator(input_bed):
                out_fd.write("%s\t%s\t%s\t%i\t%i\t.\t.\t.\tID=%s%i\n" % (line_list[0], source, feature_type,
                                                                         int(line_list[1]) + 1, int(line_list[2]),
                                                                         id_prefix, index
                                                                         ))
                index += 1

    @staticmethod
    def convert_gff_to_simple_bed(input_gff, output_bed, feature_type_list=[], scaffold_id_file=None):
        if scaffold_id_file:
            scaffolds_id_list = IdList(filename=scaffold_id_file)
        with open(input_gff, "r") as gff_fd:
            with open(output_bed, "w") as bed_fd:
                for line in gff_fd:
                    if line[0] == "#":
                        continue
                    tmp_list = line.strip().split("\t")
                    if scaffold_id_file:
                        if tmp_list[0] not in scaffolds_id_list:
                            continue
                    if feature_type_list:
                        if tmp_list[2] not in feature_type_list:
                            continue
                    bed_fd.write("%s\t%s\t%s\n" % (tmp_list[0], tmp_list[3], tmp_list[4]))

    def get_id_based_dict_from_gff(self, input_gff, id_entry="ID"):
        id_based_dict = OrderedDict()

        with open(input_gff, "r") as gff_fd:
            for line in gff_fd:
                if line[0] == "#":
                    continue
                tmp_list = line.strip().split("\t")
                description_dict = self.parse_gff_annotation_string_to_dict(tmp_list[-1])

                #print description_dict[id_entry]
                id_based_dict[description_dict[id_entry][0]] = [tmp_list[0], tmp_list[3], tmp_list[4], description_dict]

        return id_based_dict

    @staticmethod
    def get_scaffold_ids_from_gff(gff_file, out_file=None):
        scaffold_id_set = IdSet()

        with open(gff_file, "r") as gff_fd:
            for line in gff_fd:
                if line[0] == "#":
                    continue
                scaffold_id = line.split("\t")[0]
                scaffold_id_set.add(scaffold_id)

        if out_file:
            scaffold_id_set.write(out_file)

        return scaffold_id_set

    @staticmethod
    def count_per_scaffold_feature_number(gff_file, out_file=None, feature_type_list=[]):
        feature_count_dict = SynDict()

        if feature_type_list:
            def check_feature_type(feature_type):
                return feature_type in feature_type_list
        else:
            def check_feature_type(feature_type):
                return True

        with open(gff_file, "r") as gff_fd:
            for line in gff_fd:
                if line[0] == "#":
                    continue
                line_list = line.split("\t")
                if check_feature_type(line_list[2]):
                    if line_list[0] in feature_count_dict:
                        feature_count_dict[line_list[0]] += 1
                    else:
                        feature_count_dict[line_list[0]] = 1

        if out_file:
            feature_count_dict.write(out_file)

        return feature_count_dict

    @staticmethod
    def check_chunks(chunk_dir, number_of_chunks, minimum_chunk_size, separator="_",
                     chunk_filename_suffix=None, chunk_filename_prefix=None):
        chunk_files = os.listdir(chunk_dir)
        filtered_chunk_files = []
        too_small_chunks = []
        chunk_numbers = []

        suffix_length = len(chunk_filename_suffix) if chunk_filename_suffix else None
        prefix_length = len(chunk_filename_prefix) if chunk_filename_prefix else None

        for filename in chunk_files:
            filename_length = len(filename)
            full_filename = "%s/%s" % (chunk_dir, filename)

            if os.path.isdir(full_filename):
                continue

            if chunk_filename_suffix and chunk_filename_prefix:
                if (filename_length <= suffix_length) or (filename_length <= prefix_length):
                    continue
                if (filename[0:prefix_length] == chunk_filename_prefix) and (filename[-suffix_length:] == chunk_filename_suffix):
                    filtered_chunk_files.append(filename)

            elif chunk_filename_prefix:
                if filename_length <= prefix_length:
                    continue
                if filename[0:prefix_length] == chunk_filename_prefix:
                    filtered_chunk_files.append(filename)

            elif chunk_filename_suffix:
                if filename_length <= suffix_length:
                    continue
                if filename[-suffix_length:] == chunk_filename_suffix:
                    filtered_chunk_files.append(filename)
            else:
                filtered_chunk_files.append(filename)

        for filename in filtered_chunk_files:
            full_filename = "%s/%s" % (chunk_dir, filename)
            chunk_number = int(filename.strip(chunk_filename_suffix).split(separator)[-1]) if chunk_filename_suffix else int(filename.split(separator)[-1])
            chunk_numbers.append(chunk_number)

            if os.path.getsize(full_filename) < minimum_chunk_size:
                too_small_chunks.append(chunk_number)

        chunk_numbers.sort()
        too_small_chunks.sort()

        if chunk_numbers[-1] > number_of_chunks:
            print("Largest number of present chunks(%i) is larger than expected(%i)" % (chunk_numbers[-1],
                                                                                        number_of_chunks))

        absent_chunks = []
        for i in range(1, number_of_chunks + 1):
            if i not in chunk_numbers:
                absent_chunks.append(i)

        if absent_chunks:
            print("Absent chunks(total %i): " % len(absent_chunks))
            for k in absent_chunks:
                print(k)
        else:
            print("No absent chunks")

        if too_small_chunks:
            print("Too small chunks (total %i): " % len(too_small_chunks))
            for k in too_small_chunks:
                print(k)
        else:
            print("No too small chunks")

    def merge_overlapping_feature_in_simple_format(self,
                                                   input_file_file_list, scaffold_id_column,
                                                   feature_start_column, feature_end_column,
                                                   output_file=None, output_separator="\t",
                                                   comments_prefix="#", input_separator="\t",
                                                   coordinates_type="1-based", return_seqfeature_dict=False,
                                                   feature_type=None):

        file_list = [input_file_file_list] if isinstance(input_file_file_list, str) else input_file_file_list

        record_dict_list = []

        for filename in file_list:
            record_dict_list.append(OrderedDict())
            for line_list in self.file_line_as_list_generator(filename,
                                                              comments_prefix=comments_prefix,
                                                              separator=input_separator):
                if line_list[scaffold_id_column] not in record_dict_list[-1]:
                    record_dict_list[-1][line_list[scaffold_id_column]] = []
                record_dict_list[-1][line_list[scaffold_id_column]].append([(int(line_list[feature_start_column]) - 1 if coordinates_type == "1-based" else int(line_list[feature_start_column])),
                                                                            int(line_list[feature_end_column])])

        unified_dict = OrderedDict()
        merged_dict = OrderedDict()

        #print record_dict_list[0]

        scaffold_set = set()
        for record_dict in record_dict_list:
            scaffold_set |= set(record_dict.keys())

        for scaffold in scaffold_set:
            unified_dict[scaffold] = []
            merged_dict[scaffold] = []

        for record_dict in record_dict_list:
            for scaffold in record_dict:

                unified_dict[scaffold] += record_dict[scaffold]
                #print "AAAAAAAAAA"
                #print scaffold, unified_dict[scaffold], record_dict[scaffold]

        for scaffold in unified_dict:
            if unified_dict[scaffold]:
                unified_dict[scaffold].sort()
            if unified_dict[scaffold] is None:
                print(scaffold)

        #print unified_dict

        for scaffold in unified_dict:

            number_of_records = len(unified_dict[scaffold])
            if number_of_records == 0:
                continue

            # [a, b) [c, d), a < b, c < d
            # after sorting c >= a
            i = 1

            prev_coordinates = deepcopy(unified_dict[scaffold][0])

            #print scaffold, number_of_records, prev_coordinates
            #print "\t", unified_dict[scaffold]

            while i < number_of_records:
                if unified_dict[scaffold][i][0] > prev_coordinates[1]: # c > b
                    #print "AAAAAA", "\t", prev_coordinates, unified_dict[scaffold][i]
                    merged_dict[scaffold].append(deepcopy(prev_coordinates))
                    prev_coordinates = deepcopy(unified_dict[scaffold][i])

                elif unified_dict[scaffold][i][1] > prev_coordinates[1]: # d > b; c<=b
                    #print "BBBBBB", "\t",prev_coordinates, unified_dict[scaffold][i]
                    prev_coordinates[1] = deepcopy(unified_dict[scaffold][i][1])
                else: # d <= b
                    #print "CCCCCC", "\t",prev_coordinates, unified_dict[scaffold][i]
                    pass
                i += 1
            if merged_dict[scaffold]:
                if prev_coordinates != merged_dict[scaffold][-1]:
                    merged_dict[scaffold].append(prev_coordinates)
            else:
                merged_dict[scaffold].append(prev_coordinates)

            #print "\t", unified_dict[scaffold]
            #print "\t", merged_dict[scaffold]
        #print unified_dict
        #print merged_dict
        if output_file:
            with self.metaopen(output_file, "w") as out_fd:
                for scaffold in merged_dict:
                    for feature in merged_dict[scaffold]:
                        out_fd.write(output_separator.join(map(str,
                                                               [scaffold,
                                                                feature[0] + 1 if coordinates_type == "1-based" else feature[0],
                                                                feature[1]])) + "\n")

        if return_seqfeature_dict and feature_type:
            feature_dict = OrderedDict()
            for region in merged_dict:
                feature_dict[region] = []
                for (start, stop) in merged_dict[region]:
                    feature_dict[region].append(SeqFeature(FeatureLocation(start, stop),
                                                           type=feature_type,
                                                           strand=None))
            return feature_dict
        elif return_seqfeature_dict and (not feature_type):
            raise ValueError("ERROR!!! Feature type for seqfeature records was not set!")
        else:
            return merged_dict

    def rename_scaffolds_in_gff(self, input_gff, syn_file, output_prefix, verbose=True):

        syn_dict = SynDict(filename=syn_file)
        skipped_id_list = IdSet()

        output_gff = "%s.renamed.gff" % output_prefix
        skipped_gff = "%s.skipped.gff" % output_prefix
        skipped_id_file = "%s.skipped_scaffolds.ids" % output_prefix

        with self.metaopen(input_gff, "r") as in_fd, \
             self.metaopen(output_gff, "w") as out_fd, \
             self.metaopen(skipped_gff, "w") as skipped_fd:

            for line in in_fd:
                if line[0] == "#":
                    out_fd.write(line)
                gff_list = line.split("\t")
                if gff_list[0] in syn_dict:
                    gff_list[0] = syn_dict[gff_list[0]]
                    out_fd.write("\t".join(gff_list))
                else:
                    skipped_fd.write(line)
                    skipped_id_list.add(gff_list[0])

        if verbose:
            print("Not renamed scaffolds: %i" % len(skipped_id_list))

        skipped_id_list.write(skipped_id_file)

    @staticmethod
    def feature_list_entry_to_tab_str(feature_entry):
        return "%s\t%s\t%s\t%s" % (feature_entry[0],
                                   str(feature_entry[1]),
                                   str(feature_entry[2]),
                                   str(feature_entry[3]))

    @staticmethod
    def feature_list_entry_to_gatk_interval_str(feature_entry):
        return "%s:%s-%s" % (feature_entry[0],
                             str(feature_entry[1]),
                             str(feature_entry[2]))

    def get_feature_dict(self, input_gff, output_prefix=None, feature_type_list=["CDS"], unification_key="Parent"):

        feature_dict = SynDict()
        for line_list in self.file_line_as_list_generator(input_gff, comments_prefix="#", separator="\t"):
            annotation_dict = self.parse_gff_annotation_string_to_dict(line_list[self.GFF_ATTRIBUTE_COLUMN])

            if line_list[self.GFF_FEATURETYPE_COLUMN] not in feature_type_list:
                continue

            if unification_key not in annotation_dict:
                continue
            #print unification_key
            #print(annotation_dict)

            if annotation_dict[unification_key][0] not in feature_dict:
                feature_dict[annotation_dict[unification_key][0]] = []

            feature_dict[annotation_dict[unification_key][0]].append([line_list[self.GFF_SCAFFOLD_COLUMN],
                                                                     line_list[self.GFF_START_COLUMN],
                                                                     line_list[self.GFF_END_COLUMN],
                                                                     line_list[self.GFF_STRAND_COLUMN]])

        if output_prefix:
            feature_dict.write("%s.tab" % output_prefix,
                               value_expression=self.feature_list_entry_to_tab_str,
                               line_per_value=True)
            feature_dict.write("%s.coordinates_only.tab" % output_prefix,
                               value_expression=self.feature_list_entry_to_tab_str,
                               line_per_value=True,
                               values_only=True)

            feature_dict.write("%s.list" % output_prefix,
                               value_expression=self.feature_list_entry_to_gatk_interval_str,
                               line_per_value=True)
            feature_dict.write("%s.coordinates_only.list" % output_prefix,
                               value_expression=self.feature_list_entry_to_gatk_interval_str,
                               line_per_value=True,
                               values_only=True)

        return feature_dict

    def parse_regions(self, input_file, format="gff", comment_prefix="#", separator="\t", bed_format="0-based"):
        region_dict = SynDict()
        # All coordinates are converted to 0-based in python notation

        if format == "gff":
            for line in self.file_line_as_list_generator(input_file, comments_prefix="#", separator="\t"):
                if line[self.GFF_SCAFFOLD_COLUMN] not in region_dict:
                    region_dict[line[self.GFF_SCAFFOLD_COLUMN]] = [[int(line[self.GFF_START_COLUMN]) - 1,
                                                                    int(line[self.GFF_END_COLUMN])]]
                else:
                    region_dict[line[self.GFF_SCAFFOLD_COLUMN]].append([int(line[self.GFF_START_COLUMN]) - 1,
                                                                        int(line[self.GFF_END_COLUMN])])

        elif format == "bed":
            for line in self.file_line_as_list_generator(input_file, comments_prefix="#", separator="\t"):
                if line[self.BED_SCAFFOLD_COLUMN] not in region_dict:
                    region_dict[line[self.BED_SCAFFOLD_COLUMN]] = [[(int(line[self.BED_START_COLUMN]) - 1) if bed_format == "1-based" else int(line[self.BED_START_COLUMN]),
                                                                    int(line[self.BED_END_COLUMN])]]
                else:
                    region_dict[line[self.BED_SCAFFOLD_COLUMN]].append([(int(line[self.BED_START_COLUMN]) - 1) if bed_format == "1-based" else int(line[self.BED_START_COLUMN]),
                                                                        int(line[self.BED_END_COLUMN])])

        elif format == "gatk":
            pass

        return region_dict



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os
from RouToolPa.Tools.Abstract import Tool


class MaSuRCA(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "masurca", path=path, max_threads=max_threads)

    def generate_config(self, libraries_list, config_file, jellyfish_hash_size=200000000, kmer_size="auto", illumina_only_assembly=True,
                        limit_jump_coverage=None, source="eukaryota", trim_long_homopolymers=False, cgwErrorRate=0.15,
                        ovlMemory="4GB", minimum_count_kmer_in_error_correction=1):
        # Library description format
        # (library_type, mean_insert_size, stdev_insert_size, read_files_list)
        # library_types: PE, SE, MP, OTHER
        config = "# DATA is specified as type {PE,JUMP,OTHER} and 5 fields:\n"
        config += "# 1)two_letter_prefix 2)mean 3)stdev 4)fastq(.gz)_fwd_reads\n"
        config += "# 5)fastq(.gz)_rev_reads. The PE reads are always assumed to be\n"
        config += "# innies, i.e. --->.<---, and JUMP are assumed to be outties\n"
        config += "# <---.--->. If there are any jump libraries that are innies, such as\n"
        config += "# longjump, specify them as JUMP and specify NEGATIVE mean. Reverse reads\n"
        config += "# are optional for PE libraries and mandatory for JUMP libraries. Any\n"
        config += "# OTHER sequence data (454, Sanger, Ion torrent, etc) must be first\n"
        config += "# converted into Celera Assembler compatible .frg files (see\n"
        config += "# http://wgs-assembler.sourceforge.com)\n"
        config += "DATA\n"

        for lib_type, mean_insert_size, stdev_insert_size, read_files_list in libraries_list:
            if lib_type == "PE":
                config += "PE= pe %i %i %s\n" % (mean_insert_size, stdev_insert_size, " ". join(read_files_list))
            elif lib_type == "MP":
                config += "JUMP= sh %i %i %s\n" % (mean_insert_size, stdev_insert_size, " ". join(read_files_list))
            else:
                for read_file in read_files_list:
                    config += "OTHER=%s\n" % read_file
        config += "END\n"
        config += "PARAMETERS\n"
        config += "#this is k-mer size for deBruijn graph values between 25 and 101 are supported, auto will compute the optimal size based on the read data and GC content\n"
        config += "GRAPH_KMER_SIZE = %s\n" % str(kmer_size)
        config += "#set this to 1 for Illumina-only assemblies and to 0 if you have 1x or more long (Sanger, 454) reads, you can also set this to 0 for large data sets with high jumping clone coverage, e.g. >50x"
        config += "USE_LINKING_MATES = %i\n" % (1 if illumina_only_assembly else 0)
        config += "#this parameter is useful if you have too many jumping library mates. Typically set it to 60 for bacteria and 300 for the other organisms"
        config += "LIMIT_JUMP_COVERAGE = %i\n" % limit_jump_coverage if limit_jump_coverage else ""
        config += "#these are the additional parameters to Celera Assembler.  do not worry about performance, number or processors or batch sizes -- these are computed automatically."
        config += "#set cgwErrorRate=0.25 for bacteria and 0.1<=cgwErrorRate<=0.15 for other organisms."
        config += "CA_PARAMETERS = cgwErrorRate=%f ovlMemory=%s\n" % (cgwErrorRate, str(ovlMemory))
        config += "#minimum count k-mers used in error correction 1 means all k-mers are used.  one can increase to 2 if coverage >100\n"
        config += "KMER_COUNT_THRESHOLD = %i\n" % minimum_count_kmer_in_error_correction
        config += "#auto-detected number of cpus to use\n"
        config += "NUM_THREADS = %i\n" % self.threads
        config += "#this is mandatory jellyfish hash size -- a safe value is estimated_genome_size*estimated_coverage\n"
        config += "JF_SIZE = %i\n" % jellyfish_hash_size
        config += "#this specifies if we do (1) or do not (0) want to trim long runs of homopolymers (e.g. GGGGGGGG) from 3' read ends, use it for high GC genomes\n"
        config += "DO_HOMOPOLYMER_TRIM = %i\n" % (1 if trim_long_homopolymers else 0)
        config += "END\n"

        with open(config_file, "w") as out_fd:
            out_fd.write(config)

    def generate_script(self, config_file, output_script):

        options = " %s" % config_file
        options += " -o %s" % output_script

        self.execute(options)

    def assembly(self, libraries_list, config_file, output_script, jellyfish_hash_size=200000000, kmer_size="auto", illumina_only_assembly=True,
                        limit_jump_coverage=None, source="eukaryota", trim_long_homopolymers=False, cgwErrorRate=0.15,
                        ovlMemory="4GB", minimum_count_kmer_in_error_correction=1):

        self.generate_config(libraries_list=libraries_list, config_file=config_file,
                             jellyfish_hash_size=jellyfish_hash_size, kmer_size=kmer_size,
                             illumina_only_assembly=illumina_only_assembly, limit_jump_coverage=limit_jump_coverage,
                             source=source, trim_long_homopolymers=trim_long_homopolymers, cgwErrorRate=cgwErrorRate,
                             ovlMemory=ovlMemory, minimum_count_kmer_in_error_correction=minimum_count_kmer_in_error_correction)
        self.generate_script(config_file, output_script)
        self.execute("", cmd=output_script)


if __name__ == "__main__":
    pass


#!/usr/bin/env python
import os
from RouToolPa.Tools.Abstract import JavaTool



class CombineVariants(JavaTool):

    def __init__(self,  java_path="", max_threads=4, jar_path="", max_memory=None, timelog=None):
        JavaTool.__init__(self, "GenomeAnalysisTK.jar -T CombineVariants", java_path=java_path,
                          max_threads=max_threads, jar_path=jar_path, max_memory=max_memory,
                          timelog=timelog)
    # http://www.broadinstitute.org/gatk/gatkdocs/org_broadinstitute_sting_gatk_walkers_variantutils_CombineVariants.html

    def combine_from_same_source(self, reference_file, input_filelist, output_vcf):

        options = " -R %s" % reference_file
        options += " -nt %i" % self.threads
        options += " --variant %s" % (input_filelist if isinstance(input_filelist, str) else " --variant ".join(input_filelist))
        options += " --genotypemergeoption UNSORTED"
        options += " -o %s" % output_vcf

        self.execute(options=options)

    def combine_from_different_sources(self, gatk_dir, reference, output_file,
                                       file_list, merge_option="REQUIRE_UNIQUE"):
        variants_string = " ".join(list(map(lambda x: "--variant %s" % x, file_list)))

        os.system("java -Xmx2g -jar %sGenomeAnalysisTK.jar -R %s -T CombineVariants %s -o %s -genotypeMergeOptions %s"
                  % (gatk_dir, reference, variants_string, output_file, merge_option))



#!/usr/bin/env python
import os
from RouToolPa.Tools.Abstract import Tool



class Novoalign(Tool):

    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "novoalign", path=path, max_threads=max_threads)

    def index(self, index_name, reference_files, kmer_size=None, step_size=None, soft_masking=False,
              bisulfite_index=False, color_index=False, internal_name=None):

        options = ""
        options += " -k %i" % kmer_size if kmer_size else ""
        options += " -s %i" % step_size if step_size else ""
        options += " -m" if soft_masking else ""
        options += " -b" if bisulfite_index else ""
        options += " -c" if color_index else ""
        options += " -n %s" % internal_name if internal_name else ""
        options += " %s" % index_name
        options += " %s" % " ".join(reference_files) if isinstance(reference_files, list) else " %s" % reference_files

        self.execute(options, cmd="novoindex")

    def align(self, reads, index, output_file="alignment.sam", gap_extend_pen=None,
              gap_open_pen=None, out_fmt="SAM", in_fmt="ILM1.8"):
        """
        -o %s       Output format : Native, Pairwise, SAM, extended
        -F %s       Input format:
                    FA              fasta without quality values
                    SLXFQ           fastq with solexa style quality values
                    STDFQ           fastq with Sanger style quality values
                    ILMFQ           fastq with Illumina-64 style quality values
                    ILM1.8          fastq with Illumina-33 style quality values
                    PRB
                    PRBnSEQ
                    QSEQ
                    BAMSE
                    BAMPE
                    BAM
        -g %i       Gap open penalty. Default 40.
        -x %i       Gap extend penalty. Default 6.
        -d %s       full path to index
        -f %s [%s]  Files with reads
        """
        options = ""
        options += " -g %i" % gap_open_pen if gap_open_pen else ""
        #options += " -x %i" % gap_extend_pen if gap_extend_pen else ""
        options += " -o %s" % out_fmt
        options += " -F %s" % in_fmt
        options += " -d %s" % index
        options += " -f %s" % " ".join(reference) if isinstance(reads, list) else " -f %s" % reads
        options += " -c %i" % self.max_threads
        options += " > %s" % output_file
        self.execute(options, cmd="novoalign")


if __name__ == "__main__":
    workdir = "/media/mahajrod/d9e6e5ee-1bf7-4dba-934e-3f898d9611c8/Data/Alsu/reference/Alsu24mc/"
    reference = "Alsu24mc.fasta"
    index_name = "Alsu24mc.nidx"
    os.chdir(workdir)
    Novoalign = Novoalign()
    Novoalign.index(index_name, reference)



#!/usr/bin/env python
from RouToolPa.Tools.Abstract import Tool
from RouToolPa.Collections.General import SynDict



class CDHit(Tool):
    """
    T
    """

    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "cd-hit", path=path, max_threads=max_threads)

    @staticmethod
    def convert_clustering_to_fam(input_clustering_file, output_fam):
        clusering_dict = SynDict()
        with open(input_clustering_file, "r") as in_fd:
            cluster_id = in_fd.readline()[1:].strip().replace(" ", "_")
            seq_list = []
            for line in in_fd:
                if line[0] == ">":
                    clusering_dict[cluster_id] = seq_list
                    cluster_id = line[1:].strip().replace(" ", "_")
                    seq_list = []
                    continue
                seq_list.append(line.strip().split()[2][1:-3])

        clusering_dict.write(output_fam, splited_values=True)







from RouToolPa.Tools.Abstract import JavaTool


class UnifiedGenotyper(JavaTool):

    def __init__(self,  java_path="", max_threads=4, jar_path="", max_memory=None, timelog=None):
        JavaTool.__init__(self, "GenomeAnalysisTK.jar -T UnifiedGenotyper", java_path=java_path,
                          max_threads=max_threads, jar_path=jar_path, max_memory=max_memory,
                          timelog=timelog)

    def variant_call(self,
                     alignment,
                     reference_file,
                     stand_emit_conf=40,
                     stand_call_conf=100,
                     output_mode="EMIT_VARIANTS_ONLY",
                     discovery_mode="BOTH",
                     output_file="GATK_raw.vcf",
                     default_base_qualities=None,
                     quality_to_change=None,
                     quality_to_change_to=60,
                     remove_spliced_reads=None,
                     use_spliced_reads=None):


        # manual http://www.broadinstitute.org/gatk/gatkdocs/org_broadinstitute_gatk_tools_walkers_genotyper_UnifiedGenotyper.html
        # output_mode values:
        # EMIT_VARIANTS_ONLY
        # EMIT_ALL_CONFIDENT_SITES
        # EMIT_ALL_SITES

        # discovery_mode values:
        # SNP
        # INDEL
        # GENERALPLOIDYSNP
        # GENERALPLOIDYINDEL
        # BOTH

        options = " -nt %i" % self.threads
        options += " -R %s" % reference_file

        options += " -I %s" % alignment
        options += " -stand_call_conf %i" % stand_call_conf
        options += " -stand_emit_conf %i" % stand_emit_conf
        options += " -o %s" % output_file
        options += " --output_mode %s" % output_mode
        options += " -glm %s" % discovery_mode
        options += " --defaultBaseQualities %i" % default_base_qualities if default_base_qualities else ""
        options += " --filter_reads_with_N_cigar" if remove_spliced_reads else ""
        options += " -U ALLOW_N_CIGAR_READS" if use_spliced_reads else ""

        if quality_to_change and quality_to_change_to:
            options += " -rf ReassignOneMappingQuality -RMQF %i -RMQT %i" % (quality_to_change, quality_to_change_to)

        self.execute(options=options)

        """
        default_qualities = ""
        if default_base_qualities:
            default_qualities = "--defaultBaseQualities %i" % default_base_qualities

        #gatk_dir = check_path(GATK_dir)
        os.system(" java -jar %sGenomeAnalysisTK.jar -nt %i -l INFO -R %s -T UnifiedGenotyper -I %s -stand_call_conf %i -stand_emit_conf %i -o %s --output_mode %s -glm %s %s"
                  % (gatk_dir, num_of_threads, reference_file, alignment, stand_call_conf, stand_emit_conf, output_file, output_mode, discovery_mode, default_qualities))
        """



#!/usr/bin/env python
from RouToolPa.Tools.Abstract import Tool
from RouToolPa.Routines import FileRoutines

class PRANK(Tool):
    def __init__(self, path="", max_threads=1):
        Tool.__init__(self, "prank", path=path, max_threads=max_threads)

    @staticmethod
    def parse_common_options(tree_file=None, output_format=None, show_xml=None, show_tree=None,
                             show_ancestral_sequences=None, show_evolutionary_events=None,
                             showall=None, compute_posterior_support=None, njtree=None, skip_insertions=False,
                             codon_alignment=None, translated_alignment=None):
        # TODO: add rest of options
        options = " -t=%s" % tree_file if tree_file else ""
        options += " -f=%s" % output_format if output_format else ""
        options += " -showxml" if show_xml else ""
        options += " -showtree" if show_tree else ""
        options += " -showanc" if show_ancestral_sequences else ""
        options += " -showevents" if show_evolutionary_events else ""
        options += " -showall" if showall else ""
        options += " -support" if compute_posterior_support else ""
        options += " -njtree" if njtree else ""
        options += " -F" if skip_insertions else ""
        options += " -codon" if codon_alignment else ""
        options += " -translate" if translated_alignment else ""

        return options

    def align(self, sequence_file, output, tree_file=None, output_format=None, show_xml=None,
              show_tree=None, show_ancestral_sequences=None, show_evolutionary_events=None,
              showall=None, compute_posterior_support=None, njtree=None, skip_insertions=False,
              codon_alignment=None, translated_alignment=None):
        # TODO: add rest of options
        options = " -d=%s" % sequence_file
        options += " -o=%s" % output

        options += self.parse_common_options(tree_file=tree_file, output_format=output_format, show_xml=show_xml,
                                             show_tree=show_tree, show_ancestral_sequences=show_ancestral_sequences,
                                             show_evolutionary_events=show_evolutionary_events, showall=showall,
                                             compute_posterior_support=compute_posterior_support, njtree=njtree,
                                             skip_insertions=skip_insertions, codon_alignment=codon_alignment,
                                             translated_alignment=translated_alignment)
        self.execute(options)

    def parallel_align(self, list_of_files, output_directory, output_suffix=None, tree_file=None, output_format=None, show_xml=None,
                       show_tree=None, show_ancestral_sequences=None, show_evolutionary_events=None,
                       showall=None, compute_posterior_support=None, njtree=None, skip_insertions=False,
                       codon_alignment=None, translated_alignment=None,
                       cmd_log_file=None,
                       cpus_per_task=1,
                       handling_mode="local",
                       job_name=None,
                       log_prefix=None,
                       error_log_prefix=None,
                       max_jobs=None,
                       max_running_time=None,
                       max_memory_per_node=None,
                       max_memmory_per_cpu=None,
                       modules_list=None,
                       environment_variables_dict=None):

        common_options = self.parse_common_options(tree_file=tree_file, output_format=output_format, show_xml=show_xml,
                                                   show_tree=show_tree, show_ancestral_sequences=show_ancestral_sequences,
                                                   show_evolutionary_events=show_evolutionary_events, showall=showall,
                                                   compute_posterior_support=compute_posterior_support, njtree=njtree,
                                                   skip_insertions=skip_insertions, codon_alignment=codon_alignment,
                                                   translated_alignment=translated_alignment)

        FileRoutines.safe_mkdir(output_directory)
        options_list = []
        for filename in list_of_files:
            basename = FileRoutines.split_filename(filename)[1]
            op = common_options
            op += " -d=%s" % filename
            op += " -o=%s/%s.fasta" % (output_directory,
                                       ("%s_%s" % (basename, output_suffix)) if output_suffix else basename)
            options_list.append(op)
        if handling_mode == "local":
            self.parallel_execute(options_list)
        elif handling_mode == "slurm":

            cmd_list = ["%s%s %s" % ((self.path + "/") if self.path else "", self.cmd, options) for options in options_list]
            self.slurm_run_multiple_jobs_in_wrap_mode(cmd_list,
                                                      cmd_log_file,
                                                      max_jobs=max_jobs,
                                                      job_name=job_name,
                                                      log_prefix=log_prefix,
                                                      error_log_prefix=error_log_prefix,
                                                      cpus_per_node=None,
                                                      max_running_jobs=None,
                                                      max_running_time=max_running_time,
                                                      cpus_per_task=cpus_per_task,
                                                      max_memory_per_node=max_memory_per_node,
                                                      max_memmory_per_cpu=max_memmory_per_cpu,
                                                      modules_list=modules_list,
                                                      environment_variables_dict=environment_variables_dict)

    def parallel_codon_alignment(self, list_of_files, output_directory, output_suffix=None, tree_file=None, output_format=None, show_xml=None,
                                 show_tree=None, show_ancestral_sequences=None, show_evolutionary_events=None,
                                 showall=None, compute_posterior_support=None, njtree=None,
                                 cmd_log_file=None,
                                 cpus_per_task=1,
                                 handling_mode="local",
                                 job_name=None,
                                 log_prefix=None,
                                 error_log_prefix=None,
                                 max_jobs=None,
                                 max_running_time=None,
                                 max_memory_per_node=None,
                                 max_memmory_per_cpu=None,
                                 modules_list=None,
                                 environment_variables_dict=None):

        self.parallel_align(list_of_files, output_directory, output_suffix=output_suffix, tree_file=tree_file,
                            output_format=output_format, show_xml=show_xml, show_tree=show_tree,
                            show_ancestral_sequences=show_ancestral_sequences,
                            show_evolutionary_events=show_evolutionary_events, showall=showall,
                            compute_posterior_support=compute_posterior_support, njtree=njtree, skip_insertions=True,
                            codon_alignment=True,
                            cmd_log_file=cmd_log_file,
                            cpus_per_task=cpus_per_task,
                            handling_mode=handling_mode,
                            job_name=job_name,
                            log_prefix=log_prefix,
                            error_log_prefix=error_log_prefix,
                            max_jobs=max_jobs,
                            max_running_time=max_running_time,
                            max_memory_per_node=max_memory_per_node,
                            max_memmory_per_cpu=max_memmory_per_cpu,
                            modules_list=modules_list,
                            environment_variables_dict=environment_variables_dict)



#!/usr/bin/env python
from RouToolPa.Tools.Abstract import Tool


class BamToFastq(Tool):
    """
    T
    """

    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "bedtools bamtofastq", path=path, max_threads=max_threads)

    def convert(self, in_bam, out_left_fastq, out_right_fastq=None, use_bam_mate_tags=False):

        options = " -tags" if use_bam_mate_tags else ""
        options += " -i %s" % in_bam
        options += " -fq %s" % out_left_fastq
        options += " -fq2 %s" % out_right_fastq if out_right_fastq else ""

        self.execute(options)




#!/usr/bin/env python

from RouToolPa.Tools.Abstract import Tool
import RouToolPa.Formats.AlignmentFormats as AlignmentFormats


class LAST(Tool):
    """
    Class for samtools 1.0+
    Several subcommands are not implemented
    """
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "last", path=path, max_threads=max_threads)

        self.sequence_order_dict = {
                                    "input":     "0",
                                    "name":      "1",
                                    "length":    "2",
                                    "alignment": "3"
                                    }

    def parse_lastdb_options(self, db_prefix, input_fasta_list, softmasking=True, seeding_scheme="YASS",
                             verbose=True, keep_preliminary_masking=True,
                             mask_simple_repeats=True):
        options = " -P %i" % self.threads
        options += " -c" if softmasking else ""
        options += " -u %s" % seeding_scheme if seeding_scheme else ""
        options += " -R%i%i" % (1 if keep_preliminary_masking else 0,
                                1 if mask_simple_repeats else 0)

        options += " -v" if verbose else ""

        options += " %s.%s.R%i%i.%s" % (db_prefix, seeding_scheme,
                                        1 if keep_preliminary_masking else 0,
                                        1 if mask_simple_repeats else 0,
                                        "soft" if softmasking else "no_soft")
        options += " %s" % (input_fasta_list if isinstance(input_fasta_list, str) else " ".join(input_fasta_list))

        return options

    def create_last_db(self, db_prefix, input_fasta_list, softmasking=True, seeding_scheme="YASS",
                       verbose=True, keep_preliminary_masking=True, mask_simple_repeats=True):

        options = self.parse_lastdb_options(db_prefix, input_fasta_list=input_fasta_list,
                                            softmasking=softmasking,
                                            seeding_scheme=seeding_scheme, verbose=verbose,
                                            keep_preliminary_masking=keep_preliminary_masking,
                                            mask_simple_repeats=mask_simple_repeats)

        self.execute(options=options, cmd="lastdb")

    def parse_lastal_options(self, lastdb, query, output_prefix, verbose=True,
                             keep_preliminary_masking=True, mask_simple_repeats=True,
                             output_format="MAF", per_thread_memory="4G", match_score_matrix=None,
                             eg2_threshold=None, discard_limit=None):

        prefix = "%s.R%i%i.%s%s" % (output_prefix,
                                    1 if keep_preliminary_masking else 0,
                                    1 if mask_simple_repeats else 0,
                                    "EG2_%f." % eg2_threshold if eg2_threshold else "",
                                    "C_%i." % discard_limit if discard_limit else "")

        tab_filename = "%stab" % prefix
        maf_filename = "%smaf" % prefix

        options = " -P %i" % self.threads

        options += " -v" if verbose else ""
        options += " -R%i%i" % (1 if keep_preliminary_masking else 0,
                                1 if mask_simple_repeats else 0)
        options += " -f %s" % output_format if output_format else ""
        options += " -i %s" % per_thread_memory if per_thread_memory else ""
        options += " -p %s" % match_score_matrix if match_score_matrix else ""

        options += " -E %f" % eg2_threshold if eg2_threshold else ""
        options += " -C %i" % discard_limit if discard_limit else ""

        options += " %s" % lastdb
        options += " %s" % query
        if output_format == "MAF":
            options += " | tee %s | maf-convert tab > %s" % (maf_filename, tab_filename)
        else:
            options += " > %s" % tab_filename
        return options

    def parse_last_train_options(self, lastdb, query, output_prefix, verbose=True,
                                 reverse_complement_symmetry=True, directional_symmetry=True,
                                 insertion_deletion_symmetry=True, eg2_threshold=0.05,
                                 discard_limit=2):

        output = "%s.%s%strain.out" % (output_prefix,
                                       "EG2_%f." % eg2_threshold if eg2_threshold else "",
                                       "C_%i." % discard_limit if discard_limit else "")

        options = " -P %i" % self.threads

        options += " -v" if verbose else ""
        options += " --revsym" if reverse_complement_symmetry else ""
        options += " --matsym" if directional_symmetry else ""
        options += " --gapsym" if insertion_deletion_symmetry else ""
        options += " -E %f" % eg2_threshold if eg2_threshold else ""
        options += " -C %i" % discard_limit if discard_limit else ""

        options += " %s" % lastdb
        options += " %s" % query
        options += " > %s" % output

        return options

    def lastal(self, lastdb, query, output_prefix, verbose=True,
               keep_preliminary_masking=True, mask_simple_repeats=True,
               output_format="MAF", per_thread_memory="4G", match_score_matrix=None,
               eg2_threshold=None, discard_limit=None):

        options = self.parse_lastal_options(lastdb, query, output_prefix, verbose=verbose,
                                            keep_preliminary_masking=keep_preliminary_masking,
                                            mask_simple_repeats=mask_simple_repeats,
                                            output_format=output_format,
                                            per_thread_memory=per_thread_memory, match_score_matrix=match_score_matrix,
                                            eg2_threshold=eg2_threshold, discard_limit=discard_limit)

        self.execute(options=options, cmd="lastal")

    def convert_maf(self, maf_file, output_file, format="TAB"):

        options = " %s" % format
        options += " %s" % maf_file
        options += " > %s" % output_file

        self.execute(options=options, cmd="maf-convert")

    def train(self, lastdb, query, output, verbose=True,
              reverse_complement_symmetry=True, directional_symmetry=True,
              insertion_deletion_symmetry=True, eg2_threshold=0.05,
              discard_limit=2):

        options = self.parse_last_train_options(lastdb, query, output, verbose=verbose,
                                                reverse_complement_symmetry=reverse_complement_symmetry,
                                                directional_symmetry=directional_symmetry,
                                                insertion_deletion_symmetry=insertion_deletion_symmetry,
                                                eg2_threshold=eg2_threshold,
                                                discard_limit=discard_limit)

        self.execute(options=options, cmd="last-train")

    def extract_one_to_one_alignments(self, input_maf, output_prefix, max_mismap_probability=1):
        options = ""
        options += " -m %f" % max_mismap_probability if max_mismap_probability else ""
        options += " %s" % input_maf
        options += " | tee %s.first_split.maf" % output_prefix

        options += " | maf-swap | last-split"
        options += " -m %f" % max_mismap_probability if max_mismap_probability else ""
        options += " | maf-swap"
        options += " > %s.one_to_one.maf" % output_prefix

        self.execute(options, cmd="last-split")

    def filter_tab_output_by_length(self, input_tab, min_hit_len, output_tab, mode="target", verbose=False):

        input_generator = self.file_line_as_list_generator(input_tab)
        read_line_counter = 0
        written_line_counter = 0
        with self.metaopen(output_tab, "w") as out_fd:
            for line_list in input_generator:
                read_line_counter += 1
                if int(line_list[AlignmentFormats.LAST_TAB_COLS["%s_hit_len" % mode]]) >= min_hit_len:
                    out_fd.write("\t".join(line_list))
                    out_fd.write("\n")
                    written_line_counter += 1
        if verbose:
            print("Read %i lines" % read_line_counter)
            print("Written %i lines" % written_line_counter)

    def plot(self, alignment, output, first_genome_seq_id_list=None,
             second_genome_seq_id_list=None,
             first_genome_seq_order=None,
             second_genome_seq_order=None,
             xsize=None, ysize=None, ):

        options = ""

        if first_genome_seq_id_list:
            for seq_id in first_genome_seq_id_list:
                options += " -1 %s" % seq_id

        if second_genome_seq_id_list:
            for seq_id in second_genome_seq_id_list:
                options += " -2 %s" % seq_id

        options += " -x %i" % xsize
        options += " -y %i" % ysize

        if first_genome_seq_order:

            options += " --sort1 %s" % self.sequence_order_dict[first_genome_seq_order]

        if second_genome_seq_order:
            options += " --sort2 %s" % self.sequence_order_dict[second_genome_seq_order]

        options += " %s" % alignment
        options += " %s" % output

        self.execute(options=options, cmd="last-dotplot")


